# Generated by Django 3.2.9 on 2022-01-09 14:05

from django.db import migrations, models
import django.db.models.deletion
import edc_model.models.fields.other_charfield


class Migration(migrations.Migration):

    dependencies = [
        ('mocca_subject', '0025_alter_bloodresultslipid_requisition'),
    ]

    operations = [
        migrations.AddField(
            model_name='historicalpatienthealth',
            name='ph9_not_performed_reason',
            field=models.TextField(blank=True, max_length=200, null=True, verbose_name='If NO, please provide a reason'),
        ),
        migrations.AddField(
            model_name='historicalpatienthealth',
            name='ph9_performed',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No')], default='Yes', max_length=15, verbose_name='Is the PH9 assessment being performed?'),
        ),
        migrations.AddField(
            model_name='patienthealth',
            name='ph9_not_performed_reason',
            field=models.TextField(blank=True, max_length=200, null=True, verbose_name='If NO, please provide a reason'),
        ),
        migrations.AddField(
            model_name='patienthealth',
            name='ph9_performed',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No')], default='Yes', max_length=15, verbose_name='Is the PH9 assessment being performed?'),
        ),
        migrations.AlterField(
            model_name='cholinitialreview',
            name='subject_visit',
            field=models.OneToOneField(on_delete=django.db.models.deletion.PROTECT, related_name='+', to='mocca_subject.subjectvisit'),
        ),
        migrations.AlterField(
            model_name='cholmedicationadherence',
            name='other_missed_pill_reason',
            field=edc_model.models.fields.other_charfield.OtherCharField(blank=True, max_length=150, null=True, verbose_name='If other, please specify ...'),
        ),
        migrations.AlterField(
            model_name='cholreview',
            name='subject_visit',
            field=models.OneToOneField(on_delete=django.db.models.deletion.PROTECT, related_name='+', to='mocca_subject.subjectvisit'),
        ),
        migrations.AlterField(
            model_name='clinicalreview',
            name='subject_visit',
            field=models.OneToOneField(on_delete=django.db.models.deletion.PROTECT, related_name='+', to='mocca_subject.subjectvisit'),
        ),
        migrations.AlterField(
            model_name='clinicalreviewbaseline',
            name='subject_visit',
            field=models.OneToOneField(on_delete=django.db.models.deletion.PROTECT, related_name='+', to='mocca_subject.subjectvisit'),
        ),
        migrations.AlterField(
            model_name='dminitialreview',
            name='subject_visit',
            field=models.OneToOneField(on_delete=django.db.models.deletion.PROTECT, related_name='+', to='mocca_subject.subjectvisit'),
        ),
        migrations.AlterField(
            model_name='dmmedicationadherence',
            name='other_missed_pill_reason',
            field=edc_model.models.fields.other_charfield.OtherCharField(blank=True, max_length=150, null=True, verbose_name='If other, please specify ...'),
        ),
        migrations.AlterField(
            model_name='dmreview',
            name='subject_visit',
            field=models.OneToOneField(on_delete=django.db.models.deletion.PROTECT, related_name='+', to='mocca_subject.subjectvisit'),
        ),
        migrations.AlterField(
            model_name='historicalcholmedicationadherence',
            name='other_missed_pill_reason',
            field=edc_model.models.fields.other_charfield.OtherCharField(blank=True, max_length=150, null=True, verbose_name='If other, please specify ...'),
        ),
        migrations.AlterField(
            model_name='historicaldmmedicationadherence',
            name='other_missed_pill_reason',
            field=edc_model.models.fields.other_charfield.OtherCharField(blank=True, max_length=150, null=True, verbose_name='If other, please specify ...'),
        ),
        migrations.AlterField(
            model_name='historicalnextappointment',
            name='appt_date',
            field=models.DateField(blank=True, help_text='if applicable.', null=True, verbose_name='Next scheduled routine appointment'),
        ),
        migrations.AlterField(
            model_name='historicalpatienthealth',
            name='ph9appetit',
            field=models.CharField(choices=[('not_at_all', 'Not at all'), ('several_days', 'Several days'), ('more_than_half', 'More than half the days'), ('nearly_everyday', 'Nearly everyday'), ('N/A', 'Not applicable')], default='N/A', max_length=15, verbose_name='Poor appetite or over eating'),
        ),
        migrations.AlterField(
            model_name='historicalpatienthealth',
            name='ph9badabt',
            field=models.CharField(choices=[('not_at_all', 'Not at all'), ('several_days', 'Several days'), ('more_than_half', 'More than half the days'), ('nearly_everyday', 'Nearly everyday'), ('N/A', 'Not applicable')], default='N/A', max_length=15, verbose_name='Feeling bad about yourself or that you are a failure or have let yourself or your family down'),
        ),
        migrations.AlterField(
            model_name='historicalpatienthealth',
            name='ph9concen',
            field=models.CharField(choices=[('not_at_all', 'Not at all'), ('several_days', 'Several days'), ('more_than_half', 'More than half the days'), ('nearly_everyday', 'Nearly everyday'), ('N/A', 'Not applicable')], default='N/A', max_length=15, verbose_name='Trouble concentrating on things such as reading the newspapers or watching television'),
        ),
        migrations.AlterField(
            model_name='historicalpatienthealth',
            name='ph9feel',
            field=models.CharField(choices=[('not_at_all', 'Not at all'), ('several_days', 'Several days'), ('more_than_half', 'More than half the days'), ('nearly_everyday', 'Nearly everyday'), ('N/A', 'Not applicable')], default='N/A', max_length=15, verbose_name='Feeling down, depressed or hopeless'),
        ),
        migrations.AlterField(
            model_name='historicalpatienthealth',
            name='ph9functio',
            field=models.CharField(choices=[('not_at_all', 'Not at all'), ('several_days', 'Several days'), ('more_than_half', 'More than half the days'), ('nearly_everyday', 'Nearly everyday'), ('N/A', 'Not applicable')], default='N/A', max_length=15, verbose_name='If you checked off any problems on this questionnaire so far, how difficult have these problems made it for you to do your work, take care of things at home or get along with other people?'),
        ),
        migrations.AlterField(
            model_name='historicalpatienthealth',
            name='ph9interst',
            field=models.CharField(choices=[('not_at_all', 'Not at all'), ('several_days', 'Several days'), ('more_than_half', 'More than half the days'), ('nearly_everyday', 'Nearly everyday'), ('N/A', 'Not applicable')], default='N/A', max_length=15, verbose_name='Little interest or pleasure in doing things'),
        ),
        migrations.AlterField(
            model_name='historicalpatienthealth',
            name='ph9moving',
            field=models.CharField(choices=[('not_at_all', 'Not at all'), ('several_days', 'Several days'), ('more_than_half', 'More than half the days'), ('nearly_everyday', 'Nearly everyday'), ('N/A', 'Not applicable')], default='N/A', max_length=15, verbose_name='Moving or speaking so slowly  that other people could have noticed or the opposite: being so fidgety or restless that you have been moving around a lot more than usual'),
        ),
        migrations.AlterField(
            model_name='historicalpatienthealth',
            name='ph9though',
            field=models.CharField(choices=[('not_at_all', 'Not at all'), ('several_days', 'Several days'), ('more_than_half', 'More than half the days'), ('nearly_everyday', 'Nearly everyday'), ('N/A', 'Not applicable')], default='N/A', max_length=15, verbose_name='Thoughts that you would be better off dead or of hurting yourself in some way'),
        ),
        migrations.AlterField(
            model_name='historicalpatienthealth',
            name='ph9tired',
            field=models.CharField(choices=[('not_at_all', 'Not at all'), ('several_days', 'Several days'), ('more_than_half', 'More than half the days'), ('nearly_everyday', 'Nearly everyday'), ('N/A', 'Not applicable')], default='N/A', max_length=15, verbose_name='Feeling tired or having little energy'),
        ),
        migrations.AlterField(
            model_name='historicalpatienthealth',
            name='ph9troubl',
            field=models.CharField(choices=[('not_at_all', 'Not at all'), ('several_days', 'Several days'), ('more_than_half', 'More than half the days'), ('nearly_everyday', 'Nearly everyday'), ('N/A', 'Not applicable')], default='N/A', max_length=15, verbose_name='Trouble falling/staying asleep, sleeping too much'),
        ),
        migrations.AlterField(
            model_name='hivinitialreview',
            name='subject_visit',
            field=models.OneToOneField(on_delete=django.db.models.deletion.PROTECT, related_name='+', to='mocca_subject.subjectvisit'),
        ),
        migrations.AlterField(
            model_name='hivreview',
            name='subject_visit',
            field=models.OneToOneField(on_delete=django.db.models.deletion.PROTECT, related_name='+', to='mocca_subject.subjectvisit'),
        ),
        migrations.AlterField(
            model_name='htninitialreview',
            name='subject_visit',
            field=models.OneToOneField(on_delete=django.db.models.deletion.PROTECT, related_name='+', to='mocca_subject.subjectvisit'),
        ),
        migrations.AlterField(
            model_name='htnreview',
            name='subject_visit',
            field=models.OneToOneField(on_delete=django.db.models.deletion.PROTECT, related_name='+', to='mocca_subject.subjectvisit'),
        ),
        migrations.AlterField(
            model_name='nextappointment',
            name='appt_date',
            field=models.DateField(blank=True, help_text='if applicable.', null=True, verbose_name='Next scheduled routine appointment'),
        ),
        migrations.AlterField(
            model_name='patienthealth',
            name='ph9appetit',
            field=models.CharField(choices=[('not_at_all', 'Not at all'), ('several_days', 'Several days'), ('more_than_half', 'More than half the days'), ('nearly_everyday', 'Nearly everyday'), ('N/A', 'Not applicable')], default='N/A', max_length=15, verbose_name='Poor appetite or over eating'),
        ),
        migrations.AlterField(
            model_name='patienthealth',
            name='ph9badabt',
            field=models.CharField(choices=[('not_at_all', 'Not at all'), ('several_days', 'Several days'), ('more_than_half', 'More than half the days'), ('nearly_everyday', 'Nearly everyday'), ('N/A', 'Not applicable')], default='N/A', max_length=15, verbose_name='Feeling bad about yourself or that you are a failure or have let yourself or your family down'),
        ),
        migrations.AlterField(
            model_name='patienthealth',
            name='ph9concen',
            field=models.CharField(choices=[('not_at_all', 'Not at all'), ('several_days', 'Several days'), ('more_than_half', 'More than half the days'), ('nearly_everyday', 'Nearly everyday'), ('N/A', 'Not applicable')], default='N/A', max_length=15, verbose_name='Trouble concentrating on things such as reading the newspapers or watching television'),
        ),
        migrations.AlterField(
            model_name='patienthealth',
            name='ph9feel',
            field=models.CharField(choices=[('not_at_all', 'Not at all'), ('several_days', 'Several days'), ('more_than_half', 'More than half the days'), ('nearly_everyday', 'Nearly everyday'), ('N/A', 'Not applicable')], default='N/A', max_length=15, verbose_name='Feeling down, depressed or hopeless'),
        ),
        migrations.AlterField(
            model_name='patienthealth',
            name='ph9functio',
            field=models.CharField(choices=[('not_at_all', 'Not at all'), ('several_days', 'Several days'), ('more_than_half', 'More than half the days'), ('nearly_everyday', 'Nearly everyday'), ('N/A', 'Not applicable')], default='N/A', max_length=15, verbose_name='If you checked off any problems on this questionnaire so far, how difficult have these problems made it for you to do your work, take care of things at home or get along with other people?'),
        ),
        migrations.AlterField(
            model_name='patienthealth',
            name='ph9interst',
            field=models.CharField(choices=[('not_at_all', 'Not at all'), ('several_days', 'Several days'), ('more_than_half', 'More than half the days'), ('nearly_everyday', 'Nearly everyday'), ('N/A', 'Not applicable')], default='N/A', max_length=15, verbose_name='Little interest or pleasure in doing things'),
        ),
        migrations.AlterField(
            model_name='patienthealth',
            name='ph9moving',
            field=models.CharField(choices=[('not_at_all', 'Not at all'), ('several_days', 'Several days'), ('more_than_half', 'More than half the days'), ('nearly_everyday', 'Nearly everyday'), ('N/A', 'Not applicable')], default='N/A', max_length=15, verbose_name='Moving or speaking so slowly  that other people could have noticed or the opposite: being so fidgety or restless that you have been moving around a lot more than usual'),
        ),
        migrations.AlterField(
            model_name='patienthealth',
            name='ph9though',
            field=models.CharField(choices=[('not_at_all', 'Not at all'), ('several_days', 'Several days'), ('more_than_half', 'More than half the days'), ('nearly_everyday', 'Nearly everyday'), ('N/A', 'Not applicable')], default='N/A', max_length=15, verbose_name='Thoughts that you would be better off dead or of hurting yourself in some way'),
        ),
        migrations.AlterField(
            model_name='patienthealth',
            name='ph9tired',
            field=models.CharField(choices=[('not_at_all', 'Not at all'), ('several_days', 'Several days'), ('more_than_half', 'More than half the days'), ('nearly_everyday', 'Nearly everyday'), ('N/A', 'Not applicable')], default='N/A', max_length=15, verbose_name='Feeling tired or having little energy'),
        ),
        migrations.AlterField(
            model_name='patienthealth',
            name='ph9troubl',
            field=models.CharField(choices=[('not_at_all', 'Not at all'), ('several_days', 'Several days'), ('more_than_half', 'More than half the days'), ('nearly_everyday', 'Nearly everyday'), ('N/A', 'Not applicable')], default='N/A', max_length=15, verbose_name='Trouble falling/staying asleep, sleeping too much'),
        ),
    ]



from edc_adverse_event.pdf_reports import DeathReport
from edc_adverse_event.view_mixins import DeathReportListboardViewMixin
from reportlab.lib.units import cm


class CustomDeathReport(DeathReport):
    logo_data = {
        "app_label": "mocca_edc",
        "filename": "inte_logo.png",
        "first_page": (4.0 * cm, 0.83 * cm),
        "later_pages": (3.0 * cm, 0.625 * cm),
    }


class DeathReportListboardView(DeathReportListboardViewMixin):

    pdf_report_cls = CustomDeathReport



from django.apps import apps as django_apps
from django.core.exceptions import ObjectDoesNotExist


class InteCrfReportMixin:
    weight_model = "mocca_subject.followup"

    @property
    def unblinded(self):
        unblinding_request_model_cls = django_apps.get_model("mocca_prn.unblindingrequest")
        try:
            unblinded = unblinding_request_model_cls.objects.get(
                subject_identifier=self.subject_identifier, approved=True
            )
        except ObjectDoesNotExist:
            unblinded = False
        return unblinded



from django.core.exceptions import ObjectDoesNotExist
from django.template.loader import render_to_string
from django.urls import reverse
from edc_constants.constants import DEAD, NO, YES
from edc_dashboard import url_names

from ..models import (
    CareModelMixin,
    CareStatus,
    MoccaRegisterContact,
    SubjectRefusalScreening,
    SubjectScreening,
)


class ChangelistButton:
    """A base class that renders a changelist button for the MoccaRegister
    changelist.

    You need to override `context`.

    Usage:
        button = MyButton(...)
        button.rendered
    """

    template = "mocca_screening/bootstrap3/dashboard_button.html"

    def __init__(
        self,
        mocca_register=None,
        changelist_url_name=None,
    ):
        self._care_status_obj = None
        self._last_mocca_register_contact = None
        self._subject_screening_obj = None
        self._subject_screening_refusal_obj = None
        self.changelist_url_name = changelist_url_name
        self.mocca_register = mocca_register
        self.value_display = None

    def __repr__(self):
        return f"{self.__class__.__name__}(<{self.mocca_register}>)"

    def __str__(self):
        return self.rendered

    @property
    def context(self):
        """Returns the template context for the button
        or None.

        Override.
        """
        return None

    @property
    def rendered(self):
        """Returns a rendered button or a value string or None"""
        if self.context:
            return render_to_string(self.template, context=self.context)
        return self.value_display

    @property
    def subject_screening_obj(self):
        if not self._subject_screening_obj:
            try:
                self._subject_screening_obj = SubjectScreening.objects.get(
                    mocca_register=self.mocca_register
                )
            except ObjectDoesNotExist:
                pass
        return self._subject_screening_obj

    @property
    def subject_screening_refusal_obj(self):
        if not self._subject_screening_refusal_obj:
            try:
                self._subject_screening_refusal_obj = SubjectRefusalScreening.objects.get(
                    mocca_register=self.mocca_register
                )
            except ObjectDoesNotExist:
                pass
        return self._subject_screening_refusal_obj

    @property
    def care_status_obj(self):
        if not self._care_status_obj:
            try:
                self._care_status_obj = CareStatus.objects.get(
                    mocca_register=self.mocca_register
                )
            except ObjectDoesNotExist:
                pass
        return self._care_status_obj

    @property
    def last_mocca_register_contact(self):
        """Returns the last model instance from model
        MoccaRegisterContact or None.
        """
        if not self._last_mocca_register_contact:
            self._last_mocca_register_contact = (
                MoccaRegisterContact.objects.filter(mocca_register=self.mocca_register)
                .order_by("created")
                .last()
            )
        return self._last_mocca_register_contact

    @property
    def willing_or_present(self):
        """Returns True if the subject is either willing to screen
        or present today to screen.
        """
        if self.last_mocca_register_contact:
            return self.last_mocca_register_contact.willing_to_attend == YES
        return self.mocca_register.subject_present == YES

    @property
    def deceased(self):
        """Returns True if subject reported as deceased at the
        last contact.
        """
        return getattr(self.last_mocca_register_contact, "survival_status", "") == DEAD


class CareStatusButton(ChangelistButton):

    care_status_add_url_name = "mocca_screening_admin:mocca_screening_carestatus_add"
    care_status_change_url_name = "mocca_screening_admin:mocca_screening_carestatus_change"

    @property
    def context(self):
        """Returns the template context for the button
        or None.
        """

        if self.deceased:
            context = None
            self.value_display = "deceased"
        elif self.subject_screening_obj:
            context = None
        elif self.care_status_obj:
            context = self.change_button_context
        elif not self.willing_or_present:
            context = None
        else:
            context = self.add_button_context
        return context

    @property
    def add_button_context(self):
        """Returns a dictionary of additional context for the button
        if adding a model instance.
        """

        return dict(
            title=f"Add {CareStatus._meta.verbose_name}",
            url=self.add_url,
            label="Add",
            fa_icon="fas fa-plus",
            button_type="add",
        )

    @property
    def add_url(self):
        url = reverse(self.care_status_add_url_name)
        return (
            f"{url}?next={self.changelist_url_name}"
            f"&mocca_register={str(self.mocca_register.id)}"
        )

    @property
    def change_button_context(self):
        """Returns a dictionary of additional context for the button
        if editing the model instance.
        """
        return dict(
            title=f"Change {CareStatus._meta.verbose_name}",
            url=self.change_url,
            label="Edit",
            fa_icon="fas fa-pen",
            button_type="edit",
        )

    @property
    def change_url(self):
        url = reverse(self.care_status_change_url_name, args=(self.care_status_obj.id,))
        return f"{url}?next={self.changelist_url_name}"


class SubjectRefusalScreeningButton(ChangelistButton):
    refusal_add_url_name = "mocca_screening_admin:mocca_screening_subjectrefusalscreening_add"
    refusal_change_url_name = (
        "mocca_screening_admin:mocca_screening_subjectrefusalscreening_change"
    )

    @property
    def context(self):
        """Returns an url to Add/Edit the SubjectRefusalScreening
        or the empty_value string.
        """
        if self.subject_screening_refusal_obj:
            context = self.change_button_context
        elif self.subject_screening_obj:
            context = None
        elif self.deceased:
            context = None
        else:
            context = self.add_button_context
        return context

    @property
    def add_button_context(self):
        """Returns a dictionary of additional context for the button
        if adding a model instance.
        """

        return dict(
            url=self.add_url,
            label="Add",
            fa_icon="fas fa-plus",
            button_type="add",
            title=f"Add {SubjectRefusalScreening._meta.verbose_name}",
        )

    @property
    def add_url(self):
        url = reverse(self.refusal_add_url_name)
        return (
            f"{url}?next={self.changelist_url_name}"
            f"&mocca_register={str(self.mocca_register.id)}"
        )

    @property
    def change_button_context(self):
        """Returns a dictionary of additional context for the button
        if editing the model instance.
        """
        return dict(
            url=self.change_url,
            label="Edit",
            fa_icon="fas fa-pen",
            button_type="edit",
            title=f"Edit {SubjectRefusalScreening._meta.verbose_name}",
        )

    @property
    def change_url(self):
        url = reverse(
            self.refusal_change_url_name, args=(self.subject_screening_refusal_obj.id,)
        )
        return f"{url}?next={self.changelist_url_name}"


class ScreeningButton(ChangelistButton):

    screening_listboard_url_name = "screening_listboard_url"
    screening_add_url_name = "mocca_screening_admin:mocca_screening_subjectscreening_add"

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.screening_listboard_url = url_names.get(self.screening_listboard_url_name)
        self.screening_listboard_url_kwargs = dict(
            screening_identifier=self.mocca_register.screening_identifier
        )
        self.screening_add_url = reverse(self.screening_add_url_name)

    @property
    def context(self):
        """Returns the template context for the screening button
        or None.
        """
        mocca_register_contact = self.last_mocca_register_contact
        if self.subject_screening_obj:
            context = self.change_button_context
        elif self.subject_screening_refusal_obj:
            context = None
        elif not mocca_register_contact and self.mocca_register.subject_present == NO:
            context = None
        elif self.deceased:
            context = None
        elif self.willing_or_present:
            context = self.add_button_context
        else:
            context = None
        return context

    @property
    def add_button_context(self):
        """Returns a dictionary of additional context for the button
        if adding a model instance.
        """
        return dict(
            url=self.add_url,
            label="Add",
            fa_icon="fas fa-plus",
            fa_icon_after=None,
            button_type="add",
            title=f"Add {SubjectScreening._meta.verbose_name}",
        )

    @property
    def add_url(self):
        add_url = (
            f"{self.screening_add_url}?next={self.changelist_url_name}"
            f"&mocca_register={str(self.mocca_register.id)}"
        )
        return "&".join([x for x in [add_url, self.care_status_query_string] if x])

    @property
    def change_button_context(self):
        """Returns a dictionary of additional context for the button
        if editing the model instance.
        """
        return dict(
            url=self.change_url,
            label=self.mocca_register.screening_identifier,
            fa_icon="fas fa-share",
            fa_icon_after=True,
            button_type="go",
            title="Go to subject screening listboard",
        )

    @property
    def change_url(self):
        return (
            f"{reverse(self.screening_listboard_url)}?"
            f"q={self.mocca_register.screening_identifier}"
        )

    @property
    def care_status_query_string(self):
        """Return a querystring of carestatus fields=values"""
        query_string = None
        if self.care_status_obj:
            query_string = "&".join(
                [
                    f"{f}={getattr(self.care_status_obj, f) or ''}"
                    for f in [f.name for f in CareModelMixin._meta.fields]
                ]
            )
        return query_string



# Generated by Django 3.1.7 on 2021-04-13 16:42

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('mocca_screening', '0012_auto_20210412_2237'),
    ]

    operations = [
        migrations.RenameField(
            model_name='historicalmoccaregister',
            old_name='screen_now',
            new_name='subject_present',
        ),
        migrations.RenameField(
            model_name='moccaregister',
            old_name='screen_now',
            new_name='subject_present',
        ),
        migrations.AlterField(
            model_name='historicalsubjectrefusal',
            name='reason',
            field=models.CharField(choices=[('unwilling_to_say', 'Unwilling to give any reason'), ('worried_about_care', 'Worried that he/she will not get the care needed'), ('mixing_and_stigme', 'Does not want to mix with patients with other conditions because of concern around stigma'), ('mixing_and_infection', 'Does not want to mix with patients with other conditions because of concerns around contracting infections'), ('happy_vertical_care', 'Patient is happy with the care being given in vertical clinics'), ('no_participant', 'No longer wishes to be a study participant but is happy to continue receiving care in the integrated clinic'), ('receives_community_care', 'Patient is receiving HIV care in the community and does not wish to return to facility-based care'), ('unable_to_contact', 'Clinic was unable to contact the subject'), ('OTHER', 'Other, please specify')], max_length=25, verbose_name='Reason for refusal to join'),
        ),
        migrations.AlterField(
            model_name='historicalsubjectrefusalscreening',
            name='reason',
            field=models.CharField(choices=[('unwilling_to_say', 'Unwilling to give any reason'), ('worried_about_care', 'Worried that he/she will not get the care needed'), ('mixing_and_stigme', 'Does not want to mix with patients with other conditions because of concern around stigma'), ('mixing_and_infection', 'Does not want to mix with patients with other conditions because of concerns around contracting infections'), ('happy_vertical_care', 'Patient is happy with the care being given in vertical clinics'), ('no_participant', 'No longer wishes to be a study participant but is happy to continue receiving care in the integrated clinic'), ('receives_community_care', 'Patient is receiving HIV care in the community and does not wish to return to facility-based care'), ('unable_to_contact', 'Clinic was unable to contact the subject'), ('OTHER', 'Other, please specify')], max_length=25, verbose_name='Reason for refusal to screen'),
        ),
        migrations.AlterField(
            model_name='subjectrefusal',
            name='reason',
            field=models.CharField(choices=[('unwilling_to_say', 'Unwilling to give any reason'), ('worried_about_care', 'Worried that he/she will not get the care needed'), ('mixing_and_stigme', 'Does not want to mix with patients with other conditions because of concern around stigma'), ('mixing_and_infection', 'Does not want to mix with patients with other conditions because of concerns around contracting infections'), ('happy_vertical_care', 'Patient is happy with the care being given in vertical clinics'), ('no_participant', 'No longer wishes to be a study participant but is happy to continue receiving care in the integrated clinic'), ('receives_community_care', 'Patient is receiving HIV care in the community and does not wish to return to facility-based care'), ('unable_to_contact', 'Clinic was unable to contact the subject'), ('OTHER', 'Other, please specify')], max_length=25, verbose_name='Reason for refusal to join'),
        ),
        migrations.AlterField(
            model_name='subjectrefusalscreening',
            name='reason',
            field=models.CharField(choices=[('unwilling_to_say', 'Unwilling to give any reason'), ('worried_about_care', 'Worried that he/she will not get the care needed'), ('mixing_and_stigme', 'Does not want to mix with patients with other conditions because of concern around stigma'), ('mixing_and_infection', 'Does not want to mix with patients with other conditions because of concerns around contracting infections'), ('happy_vertical_care', 'Patient is happy with the care being given in vertical clinics'), ('no_participant', 'No longer wishes to be a study participant but is happy to continue receiving care in the integrated clinic'), ('receives_community_care', 'Patient is receiving HIV care in the community and does not wish to return to facility-based care'), ('unable_to_contact', 'Clinic was unable to contact the subject'), ('OTHER', 'Other, please specify')], max_length=25, verbose_name='Reason for refusal to screen'),
        ),
    ]



# Generated by Django 3.2.10 on 2021-12-09 17:36

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('mocca_ae', '0004_auto_20210428_0005'),
    ]

    operations = [
        migrations.AlterField(
            model_name='aefollowup',
            name='tracking_identifier',
            field=models.CharField(max_length=32, unique=True),
        ),
        migrations.AlterField(
            model_name='aeinitial',
            name='tracking_identifier',
            field=models.CharField(max_length=32, unique=True),
        ),
        migrations.AlterField(
            model_name='aesusar',
            name='tracking_identifier',
            field=models.CharField(max_length=32, unique=True),
        ),
        migrations.AlterField(
            model_name='aetmg',
            name='tracking_identifier',
            field=models.CharField(max_length=32, unique=True),
        ),
        migrations.AlterField(
            model_name='deathreport',
            name='tracking_identifier',
            field=models.CharField(max_length=32, unique=True),
        ),
        migrations.AlterField(
            model_name='deathreporttmg',
            name='tracking_identifier',
            field=models.CharField(max_length=32, unique=True),
        ),
        migrations.AlterField(
            model_name='historicalaefollowup',
            name='tracking_identifier',
            field=models.CharField(db_index=True, max_length=32),
        ),
        migrations.AlterField(
            model_name='historicalaeinitial',
            name='tracking_identifier',
            field=models.CharField(db_index=True, max_length=32),
        ),
        migrations.AlterField(
            model_name='historicalaesusar',
            name='tracking_identifier',
            field=models.CharField(db_index=True, max_length=32),
        ),
        migrations.AlterField(
            model_name='historicalaetmg',
            name='tracking_identifier',
            field=models.CharField(db_index=True, max_length=32),
        ),
        migrations.AlterField(
            model_name='historicaldeathreport',
            name='tracking_identifier',
            field=models.CharField(db_index=True, max_length=32),
        ),
        migrations.AlterField(
            model_name='historicaldeathreporttmg',
            name='tracking_identifier',
            field=models.CharField(db_index=True, max_length=32),
        ),
        migrations.AlterField(
            model_name='historicaldeathreporttmgsecond',
            name='tracking_identifier',
            field=models.CharField(db_index=True, max_length=32),
        ),
    ]



from django import forms
from edc_adverse_event.forms import AeInitialModelFormMixin

from ..models import AeInitial


class AeInitialForm(AeInitialModelFormMixin, forms.ModelForm):
    class Meta:
        model = AeInitial
        fields = "__all__"



from django import forms
from edc_adverse_event.forms import AeTmgModelFormMixin

from ..models import AeTmg


class AeTmgForm(AeTmgModelFormMixin, forms.ModelForm):
    class Meta:
        model = AeTmg
        fields = "__all__"



# Generated by Django 3.2.11 on 2022-01-31 16:25

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('mocca_subject', '0028_auto_20220109_1722'),
    ]

    operations = [
        migrations.AlterField(
            model_name='historicalsubjectvisitmissed',
            name='contact_attempts_count',
            field=models.IntegerField(blank=True, help_text='Not including pre-appointment reminders', null=True, validators=[django.core.validators.MinValueValidator(1)], verbose_name='Number of attempts made to contact participantsince the expected appointment date'),
        ),
        migrations.AlterField(
            model_name='subjectvisitmissed',
            name='contact_attempts_count',
            field=models.IntegerField(blank=True, help_text='Not including pre-appointment reminders', null=True, validators=[django.core.validators.MinValueValidator(1)], verbose_name='Number of attempts made to contact participantsince the expected appointment date'),
        ),
    ]



from django import forms
from edc_constants.constants import YES
from edc_crf.forms import CrfFormValidatorMixin
from edc_crf.modelform_mixins import CrfModelFormMixin
from edc_form_validators.form_validator import FormValidator
from edc_model.utils import estimated_date_from_ago, model_exists_or_raise
from edc_visit_schedule.utils import raise_if_not_baseline

from ..models import ClinicalReviewBaseline, ComplicationsBaseline


class ComplicationsBaselineFormValidator(CrfFormValidatorMixin, FormValidator):
    def clean(self):
        raise_if_not_baseline(self.cleaned_data.get("subject_visit"))
        model_exists_or_raise(
            self.cleaned_data.get("subject_visit"), model_cls=ClinicalReviewBaseline
        )
        self.required_if(YES, field="stroke", field_required="stroke_ago")
        estimated_date_from_ago(data=self.cleaned_data, ago_field="stroke_ago")
        self.required_if(YES, field="heart_attack", field_required="heart_attack_ago")
        estimated_date_from_ago(data=self.cleaned_data, ago_field="heart_attack_ago")
        self.required_if(YES, field="renal_disease", field_required="renal_disease_ago")
        estimated_date_from_ago(data=self.cleaned_data, ago_field="renal_disease_ago")
        self.required_if(YES, field="vision", field_required="vision_ago")
        estimated_date_from_ago(data=self.cleaned_data, ago_field="vision_ago")
        self.required_if(YES, field="numbness", field_required="numbness_ago")
        estimated_date_from_ago(data=self.cleaned_data, ago_field="numbness_ago")
        self.required_if(YES, field="foot_ulcers", field_required="foot_ulcers_ago")
        estimated_date_from_ago(data=self.cleaned_data, ago_field="foot_ulcers_ago")
        self.required_if(YES, field="complications", field_required="complications_other")


class ComplicationsBaselineForm(CrfModelFormMixin, forms.ModelForm):

    form_validator_cls = ComplicationsBaselineFormValidator

    class Meta:
        model = ComplicationsBaseline
        fields = "__all__"



from edc_identifier.managers import SubjectIdentifierManager
from edc_model.models import BaseUuidModel, HistoricalRecords
from edc_visit_schedule.model_mixins import CurrentSiteManager, OnScheduleModelMixin


class OnSchedule(OnScheduleModelMixin, BaseUuidModel):
    """A model used by the system. Auto-completed by subject_consent."""

    on_site = CurrentSiteManager()

    objects = SubjectIdentifierManager()

    history = HistoricalRecords()

    class Meta(OnScheduleModelMixin.Meta):
        pass



from django import forms
from edc_form_validators import INVALID_ERROR, FormValidator, FormValidatorMixin
from edc_sites import get_current_country
from edc_utils import age

from mocca_screening.mocca_original_sites import get_mocca_sites_by_country

from ..models import MoccaRegister
from ..stubs import MoccaRegisterFormValidatorStub


class MoccaRegisterFormValidator(FormValidator):
    def clean(self):
        self.validate_mocca_site()
        self.validate_dob_and_age()
        self.validate_best_tel()

    def validate_mocca_site(self):
        mocca_country = get_current_country()
        if self.cleaned_data.get("mocca_site"):
            sites = get_mocca_sites_by_country(country=mocca_country)
            if self.cleaned_data.get("mocca_site").name not in [
                v.name for v in sites.values()
            ]:
                raise forms.ValidationError(
                    {"mocca_site": "Invalid site for selected country"}
                )

    def validate_dob_and_age(self):
        if (
            self.cleaned_data.get("dob")
            and self.instance.report_datetime
            and self.get_age() < 18
        ):
            raise forms.ValidationError({"dob": "Must 18 or older"})

        if self.cleaned_data.get("dob") and not self.cleaned_data.get("birth_year"):
            raise forms.ValidationError({"birth_year": "This field is required"})

        if self.cleaned_data.get("dob") and self.cleaned_data.get("birth_year"):
            if self.cleaned_data.get("dob").year != self.cleaned_data.get("birth_year"):
                raise forms.ValidationError(
                    {"birth_year": "Must match year in date of birth."}
                )
        if (
            self.cleaned_data.get("dob")
            and self.cleaned_data.get("age_in_years")
            and self.get_age() != self.cleaned_data.get("age_in_years")
        ):
            raise forms.ValidationError(
                {
                    "age_in_years": (
                        "Must match years from given date of birth "
                        f"until now (hint: got {self.get_age()})"
                    )
                }
            )

    def get_age(self: MoccaRegisterFormValidatorStub):
        return age(
            born=self.cleaned_data.get("dob"),
            reference_dt=self.instance.report_datetime,
        ).years

    def validate_best_tel(self):

        if self.cleaned_data.get("tel_two") and not self.cleaned_data.get("tel_one"):
            self.raise_validation_error(
                {"tel_one": "This field is required"}, error_code=INVALID_ERROR
            )
        if self.cleaned_data.get("tel_three") and not self.cleaned_data.get("tel_two"):
            self.raise_validation_error(
                {"tel_one": "This field is required"}, error_code=INVALID_ERROR
            )
        condition = (
            self.cleaned_data.get("tel_one")
            or self.cleaned_data.get("tel_two")
            or self.cleaned_data.get("tel_three")
        )
        self.required_if_true(condition, field_required="best_tel")
        if (
            not self.cleaned_data.get("tel_two")
            and self.cleaned_data.get("best_tel")
            and self.cleaned_data.get("best_tel") == "tel_2"
        ):
            self.raise_validation_error(
                {"best_tel": "Invalid selection"}, error_code=INVALID_ERROR
            )
        if (
            not self.cleaned_data.get("tel_three")
            and self.cleaned_data.get("best_tel")
            and self.cleaned_data.get("best_tel") == "tel_3"
        ):
            self.raise_validation_error(
                {"best_tel": "Invalid selection"}, error_code=INVALID_ERROR
            )


class MoccaRegisterForm(FormValidatorMixin, forms.ModelForm):

    form_validator_cls = MoccaRegisterFormValidator

    screening_identifier = forms.CharField(
        label="Screening identifier",
        widget=forms.TextInput(attrs={"readonly": "readonly"}),
    )

    class Meta:
        model = MoccaRegister
        fields = "__all__"



from django.forms import ModelForm
from edc_crf.modelform_mixins import CrfModelFormMixin
from edc_visit_tracking.form_validators import VisitMissedFormValidator

from ..models import SubjectVisitMissed


class SubjectVisitMissedForm(CrfModelFormMixin, ModelForm):

    form_validator_cls = VisitMissedFormValidator

    class Meta:
        model = SubjectVisitMissed
        fields = "__all__"



from django.core.validators import (
    MaxLengthValidator,
    MaxValueValidator,
    MinLengthValidator,
    MinValueValidator,
    RegexValidator,
)
from django.db import models
from django.db.models import Index, UniqueConstraint
from django_crypto_fields.fields import (
    EncryptedCharField,
    EncryptedTextField,
    FirstnameField,
    LastnameField,
)
from edc_constants.choices import ALIVE_DEAD_UNKNOWN, GENDER, YES_NO
from edc_constants.constants import NO, UNKNOWN, YES
from edc_model.models import BaseUuidModel, HistoricalRecords
from edc_sites.models import CurrentSiteManager, SiteModelMixin
from edc_utils import get_utcnow

from mocca_lists.models import MoccaOriginalSites

from ..mocca_original_sites import get_mocca_site_limited_to

TEL_CHOICES = (
    ("tel_1", "Tel/Mobile(1)"),
    ("tel_2", "Tel/Mobile(2)"),
    ("tel_3", "Tel/Mobile(3)"),
)


class Manager(models.Manager):
    """A manager class for Crf models, models that have an FK to
    the visit model.
    """

    use_in_migrations = True

    def get_by_natural_key(self, mocca_study_identifier):
        return self.get(mocca_study_identifier=mocca_study_identifier)


class MoccaRegister(SiteModelMixin, BaseUuidModel):

    report_datetime = models.DateTimeField(default=get_utcnow)

    screening_identifier = models.CharField(
        verbose_name="MOCCA (ext) screening identifier",
        max_length=15,
        null=True,
    )

    mocca_screening_identifier = models.CharField(
        verbose_name="MOCCA (original) screening identifier",
        max_length=15,
        null=True,
        blank=True,
        help_text="If known",
    )

    mocca_study_identifier = models.CharField(
        verbose_name="MOCCA (original) study identifier",
        max_length=25,
        validators=[
            RegexValidator(
                r"0[0-9]{1}\-0[0-9]{3}|[0-9]{6}",
                "Invalid format. Expected 12-3456 for UG, 123456 for TZ",
            )
        ],
        help_text="Format must match original identifier. e.g. 12-3456 for UG, 123456 for TZ",
    )

    mocca_country = models.CharField(
        max_length=25, choices=(("uganda", "Uganda"), ("tanzania", "Tanzania"))
    )

    mocca_site = models.ForeignKey(
        MoccaOriginalSites,
        on_delete=models.PROTECT,
        limit_choices_to=get_mocca_site_limited_to,
    )

    first_name = FirstnameField(null=True)

    last_name = LastnameField(null=True)

    initials = EncryptedCharField(
        validators=[
            RegexValidator("[A-Z]{1,3}", "Invalid format"),
            MinLengthValidator(2),
            MaxLengthValidator(3),
        ],
        help_text="Use UPPERCASE letters only. May be 2 or 3 letters.",
        null=True,
        blank=False,
    )
    gender = models.CharField(max_length=10, choices=GENDER, null=True, blank=False)

    age_in_years = models.IntegerField(
        validators=[MinValueValidator(18), MaxValueValidator(110)],
        null=True,
        blank=False,
    )

    birth_year = models.IntegerField(
        validators=[MinValueValidator(1900), MaxValueValidator(2002)],
        null=True,
        blank=False,
    )
    dob = models.DateField(null=True, blank=True)

    survival_status = models.CharField(
        max_length=25, choices=ALIVE_DEAD_UNKNOWN, default=UNKNOWN
    )

    contact_attempts = models.IntegerField(default=0, help_text="auto-updated")

    call = models.CharField(verbose_name="Call?", max_length=15, choices=YES_NO, default=YES)

    subject_present = models.CharField(
        verbose_name="Patient is present. Screen now instead of calling?",
        max_length=15,
        choices=YES_NO,
        default=NO,
        help_text="Only select 'yes' if the patient is present in the clinic now.",
    )

    date_last_called = models.DateField(null=True, help_text="auto-updated")

    next_appt_date = models.DateField(
        verbose_name="Appt", null=True, blank=True, help_text="auto-updated"
    )

    notes = EncryptedTextField(verbose_name="General notes", null=True, blank=True)

    tel_one = EncryptedCharField("Tel/Mobile(1)", max_length=15, null=True)
    tel_two = EncryptedCharField("Tel/Mobile(2)", max_length=15, null=True)
    tel_three = EncryptedCharField("Tel/Mobile(3)", max_length=15, null=True)
    best_tel = models.CharField(
        verbose_name="Prefered Telephone / Mobile",
        max_length=15,
        choices=TEL_CHOICES,
        null=True,
        blank=True,
        help_text="If any, select the best telephone/mobile from above",
    )

    on_site = CurrentSiteManager()
    objects = Manager()
    history = HistoricalRecords()

    def __str__(self):
        return (
            f"{self.mocca_study_identifier} {self.initials} {self.age_in_years} {self.gender}"
        )

    def save(self, *args, **kwargs):
        if self.screening_identifier:
            self.call = NO
        super().save(*args, **kwargs)

    def natural_key(self):
        return (self.mocca_study_identifier,)

    natural_key.dependencies = [
        "sites.Site",
        "mocca_lists.MoccaOriginalSites",
    ]

    class Meta(BaseUuidModel.Meta):
        verbose_name = "MOCCA Patient Register"
        verbose_name_plural = "MOCCA Patient Register"
        ordering = ["mocca_country", "mocca_site"]
        indexes = [
            Index(fields=["mocca_country", "mocca_site"]),
            Index(fields=["mocca_study_identifier", "initials", "gender"]),
        ]
        constraints = [
            UniqueConstraint(
                fields=["mocca_screening_identifier"],
                name="unique_mocca_screening_identifier",
            ),
            UniqueConstraint(
                fields=["mocca_study_identifier"], name="unique_mocca_study_identifier"
            ),
        ]



from django import forms
from django.test import TestCase, tag
from edc_appointment.constants import INCOMPLETE_APPT
from edc_constants.constants import INCOMPLETE, NO, NOT_APPLICABLE, YES
from edc_dx_review.constants import HIV_CLINIC
from edc_reportable import GRADE3, MILLIMOLES_PER_LITER
from edc_utils import get_utcnow
from model_bakery import baker

from mocca_subject.forms import BloodResultsLipidFormValidator
from mocca_subject.forms.indicators_form import IndicatorsFormValidator

from ..mocca_test_case_mixin import MoccaTestCaseMixin


@tag("ldl")
class TestIndicators(MoccaTestCaseMixin, TestCase):
    def setUp(self):
        super().setUp()
        self.subject_screening = self.get_subject_screening(
            report_datetime=get_utcnow(), clinic_type=HIV_CLINIC
        )
        self.subject_consent = self.get_subject_consent(
            subject_screening=self.subject_screening, clinic_type=HIV_CLINIC
        )
        self.subject_visit = self.get_subject_visit(
            subject_screening=self.subject_screening,
            subject_consent=self.subject_consent,
        )
        baker.make("mocca_subject.clinicalreviewbaseline", subject_visit=self.subject_visit)

    def test_blood_lipids_form_baseline(self):
        data = {
            "subject_visit": self.subject_visit,
            "report_datetime": self.subject_visit.report_datetime,
            "crf_status": INCOMPLETE,
            "ldl": 10000,
            "ldl_units": MILLIMOLES_PER_LITER,
            "ldl_abnormal": YES,
            "ldl_reportable": GRADE3,
        }
        form_validator = BloodResultsLipidFormValidator(cleaned_data=data)
        try:
            form_validator.validate()
        except forms.ValidationError:
            pass
        self.assertDictEqual({}, form_validator._errors)

    def test_blood_lipids_form_chol_g3_baseline(self):
        data = {
            "subject_visit": self.subject_visit,
            "report_datetime": self.subject_visit.report_datetime,
            "crf_status": INCOMPLETE,
            "chol": 8.0,
            "chol_units": MILLIMOLES_PER_LITER,
            "chol_abnormal": YES,
            "chol_reportable": GRADE3,
        }
        form_validator = BloodResultsLipidFormValidator(cleaned_data=data)
        try:
            form_validator.validate()
        except forms.ValidationError:
            pass
        self.assertDictEqual({}, form_validator._errors)

    def test_blood_lipids_form_chol_normal_baseline(self):
        data = {
            "subject_visit": self.subject_visit,
            "report_datetime": self.subject_visit.report_datetime,
            "crf_status": INCOMPLETE,
            "chol": 6.0,
            "chol_units": MILLIMOLES_PER_LITER,
            "chol_abnormal": NO,
            "chol_reportable": NOT_APPLICABLE,
        }
        form_validator = BloodResultsLipidFormValidator(cleaned_data=data)
        try:
            form_validator.validate()
        except forms.ValidationError:
            pass
        self.assertDictEqual({}, form_validator._errors)

    def test_blood_lipids_form(self):
        self.subject_visit.appointment.appt_status = INCOMPLETE_APPT
        self.subject_visit.appointment.save()
        self.subject_visit.appointment.refresh_from_db()

        subject_visit = self.get_subject_visit(
            subject_screening=self.subject_screening,
            subject_consent=self.subject_consent,
            visit_code=self.subject_visit.appointment.next.visit_code,
        )
        data = {
            "subject_visit": subject_visit,
            "report_datetime": subject_visit.report_datetime,
            "crf_status": INCOMPLETE,
            "weight": None,
            "height": None,
        }
        form_validator = IndicatorsFormValidator(cleaned_data=data)
        try:
            form_validator.validate()
        except forms.ValidationError:
            pass
        self.assertNotIn("weight", form_validator._errors)
        self.assertNotIn("height", form_validator._errors)



from django.conf import settings
from django.db import models
from django.utils.html import format_html
from edc_constants.choices import YES_NO
from edc_constants.constants import NOT_APPLICABLE
from edc_dx_review.model_mixins import (
    HivArvInitiationModelMixin,
    HivArvMonitoringModelMixin,
    InitialReviewModelMixin,
)
from edc_model import models as edc_models

from ..choices import CARE_ACCESS
from ..model_mixins import CrfModelMixin, DiagnosisLocationModelMixin


class HivInitialReview(
    HivArvInitiationModelMixin,
    HivArvMonitoringModelMixin,
    InitialReviewModelMixin,
    DiagnosisLocationModelMixin,
    CrfModelMixin,
    edc_models.BaseUuidModel,
):

    subject_visit = models.OneToOneField(
        settings.SUBJECT_VISIT_MODEL,
        on_delete=models.PROTECT,
        related_name="+",
    )

    receives_care = models.CharField(
        verbose_name="Is the patient receiving care for HIV?",
        max_length=15,
        choices=YES_NO,
    )

    clinic = models.CharField(
        verbose_name="Where does the patient receive care for HIV",
        max_length=15,
        choices=CARE_ACCESS,
        default=NOT_APPLICABLE,
    )

    clinic_other = models.CharField(
        verbose_name=format_html(
            "If <u>not</u> attending here, where does the patient attend?"
        ),
        max_length=50,
        null=True,
        blank=True,
    )

    class Meta(CrfModelMixin.Meta, edc_models.BaseUuidModel.Meta):
        verbose_name = "HIV Initial Review"
        verbose_name_plural = "HIV Initial Reviews"



from django.conf import settings
from django.db import models
from edc_constants.choices import YES_NO_NA
from edc_constants.constants import NOT_APPLICABLE
from edc_model import models as edc_models
from edc_vitals import models as edc_vitals_models

from mocca_subject.choices import HTN_MANAGEMENT

from ..model_mixins import CrfModelMixin
from .model_mixins import ReviewModelMixin


class HtnReview(ReviewModelMixin, CrfModelMixin, edc_models.BaseUuidModel):

    subject_visit = models.OneToOneField(
        settings.SUBJECT_VISIT_MODEL,
        on_delete=models.PROTECT,
        related_name="+",
    )

    test_date = models.DateField(
        verbose_name="Date tested for Hypertension",
        null=True,
        blank=True,
        editable=False,
        help_text="",
    )

    dx = models.CharField(
        verbose_name="Has the patient been diagnosed with Hypertension?",
        max_length=15,
        choices=YES_NO_NA,
        default=NOT_APPLICABLE,
    )

    sys_blood_pressure = edc_vitals_models.SystolicPressureField(null=True, blank=True)

    dia_blood_pressure = edc_vitals_models.DiastolicPressureField(null=True, blank=True)

    managed_by = models.CharField(
        verbose_name="How will the patient's hypertension be managed going forward?",
        max_length=25,
        choices=HTN_MANAGEMENT,
        default=NOT_APPLICABLE,
    )

    care_start_date = models.DateField(
        verbose_name="Date clinical management started",
        null=True,
        blank=True,
    )

    class Meta(CrfModelMixin.Meta, edc_models.BaseUuidModel.Meta):
        verbose_name = "Hypertension Review"
        verbose_name_plural = "Hypertension Review"



from django.contrib import admin
from django_audit_fields.admin import audit_fieldset_tuple
from edc_crf.admin import crf_status_fieldset_tuple
from edc_form_label.form_label_modeladmin_mixin import FormLabelModelAdminMixin
from edc_model_admin import SimpleHistoryAdmin

from ..admin_site import mocca_subject_admin
from ..forms import HtnReviewForm
from ..models import HtnReview
from .modeladmin_mixins import CrfModelAdminMixin


@admin.register(HtnReview, site=mocca_subject_admin)
class HtnReviewAdmin(CrfModelAdminMixin, FormLabelModelAdminMixin, SimpleHistoryAdmin):

    form = HtnReviewForm

    fieldsets = (
        (None, {"fields": ("subject_visit", "report_datetime")}),
        ("Care", {"fields": ("managed_by", "care_delivery", "care_delivery_other")}),
        (
            "Blood Pressure Measurement",
            {
                "fields": (
                    "sys_blood_pressure",
                    "dia_blood_pressure",
                )
            },
        ),
        crf_status_fieldset_tuple,
        audit_fieldset_tuple,
    )

    radio_fields = {
        "dx": admin.VERTICAL,
        "care_delivery": admin.VERTICAL,
        "crf_status": admin.VERTICAL,
        "managed_by": admin.VERTICAL,
    }



from edc_lab.model_mixins import RequisitionModelMixin
from edc_model import models as edc_models
from edc_reference.model_mixins import ReferenceModelMixin


class SubjectRequisition(RequisitionModelMixin, ReferenceModelMixin, edc_models.BaseUuidModel):
    class Meta(RequisitionModelMixin.Meta, edc_models.BaseUuidModel.Meta):
        pass



from django.contrib import admin
from django.utils.translation import gettext_lazy as _
from edc_constants.constants import NO, YES


class ScreenedListFilter(admin.SimpleListFilter):
    title = _("Screened")

    parameter_name = "screened"

    def lookups(self, request, model_admin):
        return (
            (YES, _("Yes")),
            (NO, _("No")),
        )

    def queryset(self, request, queryset):
        if self.value() == YES:
            return queryset.filter(screening_identifier__isnull=False)
        if self.value() == NO:
            return queryset.filter(screening_identifier__isnull=True)


class CallListFilter(admin.SimpleListFilter):
    title = _("Call")

    parameter_name = "call_patient"

    def lookups(self, request, model_admin):
        return (
            (YES, _("Yes")),
            (NO, _("No")),
        )

    def queryset(self, request, queryset):
        if self.value() == YES:
            return queryset.filter(screening_identifier__isnull=True, call=YES)
        if self.value() == NO:
            return queryset.filter(call=NO)


class ContactAttemptsListFilter(admin.SimpleListFilter):
    title = _("Contact")

    parameter_name = "contact"

    def lookups(self, request, model_admin):
        return (
            (0, _("0 attempts")),
            (1, _("1 attempt")),
            (2, _("2 attempts")),
            (3, _("3 attempts")),
            (4, _("4+ attempts")),
        )

    def queryset(self, request, queryset):
        if self.value() is not None:
            if int(self.value()) == 0:
                return queryset.filter(contact_attempts=0)
            if 0 < int(self.value()) <= 3:
                return queryset.filter(contact_attempts=int(self.value()))
            if int(self.value()) > 3:
                return queryset.filter(contact_attempts__gte=int(self.value()))



from django.conf import settings
from django.db import models
from edc_constants.constants import NOT_APPLICABLE
from edc_model import models as edc_models

from ..choices import CHOL_MANAGEMENT
from ..model_mixins import CrfModelMixin
from .model_mixins import ReviewModelMixin


class CholReview(ReviewModelMixin, CrfModelMixin, edc_models.BaseUuidModel):

    subject_visit = models.OneToOneField(
        settings.SUBJECT_VISIT_MODEL,
        on_delete=models.PROTECT,
        related_name="+",
    )

    managed_by = models.CharField(
        verbose_name="How will the patient's High Cholesterol be managed going forward?",
        max_length=25,
        choices=CHOL_MANAGEMENT,
        default=NOT_APPLICABLE,
    )

    class Meta(CrfModelMixin.Meta, edc_models.BaseUuidModel.Meta):
        verbose_name = "High Cholesterol Review"
        verbose_name_plural = "High Cholesterol Review"



# Generated by Django 3.0.9 on 2020-12-06 18:40

import django.core.validators
import edc_model_fields.fields.other_charfield
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("mocca_screening", "0021_auto_20201206_2122"),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="care",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("unknown", "Unknown")],
                default="-",
                max_length=25,
                verbose_name="Is the patient in care?",
            ),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="care_facility_location",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                help_text="Either integrated care or vertical care",
                max_length=25,
                verbose_name="Does the patient receive care in this facility",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="icc",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                help_text="Either at this facility or elsewhere",
                max_length=25,
                verbose_name="Does the patient currently receive regular integrated care",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="icc_not_in_reason",
            field=models.CharField(
                blank=True,
                choices=[
                    (
                        "icc_not_available",
                        "ICC not available (or closed) in this facility",
                    ),
                    ("moved", "Moved out of area"),
                    (
                        "dont_want",
                        "Personally chose not to continue with integrated care",
                    ),
                    (
                        "advised_to_vertical",
                        "Healthcare staff asked patient to return to vertical care",
                    ),
                    ("referred_out", "Referred to another facility without an ICC"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=25,
                verbose_name="If not integrated care, why not?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="icc_since_mocca",
            field=models.CharField(
                choices=[
                    ("Yes", "Yes"),
                    ("partially", "Partially"),
                    ("No", "No"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=25,
                verbose_name="Has the patient received regular integrated care since the leaving the MOCCA (orig) trial until now?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="icc_since_mocca_comment",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                null=True,
                verbose_name="If patient has not received regular integrated care, please explain",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="mocca_participant",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                default="Yes",
                max_length=25,
                verbose_name="Was the patient enrolled to the original MOCCA study?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="mocca_study_identifier",
            field=models.CharField(
                db_index=True,
                help_text="Format must match original identifier. e.g. 12-3456 for UG, 123456 for TZ",
                max_length=25,
                validators=[
                    django.core.validators.RegexValidator(
                        "0[0-9]{1}\\-0[0-9]{3}|[0-9]{6}",
                        "Invalid format. Expected 12-3456 for UG, 123456 for TZ",
                    )
                ],
                verbose_name="MOCCA (original) study identifier",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="care",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("unknown", "Unknown")],
                default="-",
                max_length=25,
                verbose_name="Is the patient in care?",
            ),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="care_facility_location",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                help_text="Either integrated care or vertical care",
                max_length=25,
                verbose_name="Does the patient receive care in this facility",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="icc",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                help_text="Either at this facility or elsewhere",
                max_length=25,
                verbose_name="Does the patient currently receive regular integrated care",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="icc_not_in_reason",
            field=models.CharField(
                blank=True,
                choices=[
                    (
                        "icc_not_available",
                        "ICC not available (or closed) in this facility",
                    ),
                    ("moved", "Moved out of area"),
                    (
                        "dont_want",
                        "Personally chose not to continue with integrated care",
                    ),
                    (
                        "advised_to_vertical",
                        "Healthcare staff asked patient to return to vertical care",
                    ),
                    ("referred_out", "Referred to another facility without an ICC"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=25,
                verbose_name="If not integrated care, why not?",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="icc_since_mocca",
            field=models.CharField(
                choices=[
                    ("Yes", "Yes"),
                    ("partially", "Partially"),
                    ("No", "No"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=25,
                verbose_name="Has the patient received regular integrated care since the leaving the MOCCA (orig) trial until now?",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="icc_since_mocca_comment",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                null=True,
                verbose_name="If patient has not received regular integrated care, please explain",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="mocca_participant",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                default="Yes",
                max_length=25,
                verbose_name="Was the patient enrolled to the original MOCCA study?",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="mocca_study_identifier",
            field=models.CharField(
                help_text="Format must match original identifier. e.g. 12-3456 for UG, 123456 for TZ",
                max_length=25,
                unique=True,
                validators=[
                    django.core.validators.RegexValidator(
                        "0[0-9]{1}\\-0[0-9]{3}|[0-9]{6}",
                        "Invalid format. Expected 12-3456 for UG, 123456 for TZ",
                    )
                ],
                verbose_name="MOCCA (original) study identifier",
            ),
        ),
    ]



from edc_model import models as edc_models
from edc_phq9.model_mixins import Phq9ModelMixin

from ..model_mixins import CrfModelMixin


class PatientHealth(Phq9ModelMixin, CrfModelMixin, edc_models.BaseUuidModel):
    class Meta(Phq9ModelMixin.Meta, CrfModelMixin.Meta, edc_models.BaseUuidModel.Meta):
        pass



from edc_consent.constants import CLINIC_NUMBER, HOSPITAL_NUMBER
from edc_constants.constants import MOBILE_NUMBER, OTHER

IDENTITY_TYPE = (
    ("country_id", "Country ID number"),
    ("drivers", "Driver's license"),
    ("passport", "Passport"),
    (CLINIC_NUMBER, "Clinic number"),
    (HOSPITAL_NUMBER, "Hospital number"),
    (MOBILE_NUMBER, "Mobile number"),
    (OTHER, "Other"),
)



from edc_adverse_event.form_validator_mixins import (
    RequiresDeathReportFormValidatorMixin,
)
from edc_consent.constants import CONSENT_WITHDRAWAL
from edc_constants.constants import DEAD, OTHER
from edc_form_validators import FormValidator
from edc_ltfu.constants import LOST_TO_FOLLOWUP
from edc_ltfu.forms import LtfuFormValidatorMixin
from edc_ltfu.utils import get_ltfu_model_name
from edc_offstudy.constants import OTHER_RX_DISCONTINUATION


class EndOfStudyFormValidator(
    LtfuFormValidatorMixin, RequiresDeathReportFormValidatorMixin, FormValidator
):

    offschedule_reason_field = "offschedule_reason"
    loss_to_followup_model = get_ltfu_model_name()
    loss_to_followup_date_field = "ltfu_date"
    loss_to_followup_reason = LOST_TO_FOLLOWUP

    def clean(self):

        self.validate_ltfu()

        self.validate_death_report_if_deceased()

        if self.cleaned_data.get("offschedule_reason"):
            if self.cleaned_data.get("offschedule_reason").name != OTHER:
                self.validate_other_specify(
                    field="offschedule_reason",
                    other_specify_field="other_offschedule_reason",
                    other_stored_value=OTHER_RX_DISCONTINUATION,
                )

            if self.cleaned_data.get("offschedule_reason").name != OTHER_RX_DISCONTINUATION:
                self.validate_other_specify(
                    field="offschedule_reason",
                    other_specify_field="other_offschedule_reason",
                    other_stored_value=OTHER,
                )

        self.required_if(DEAD, field="offschedule_reason", field_required="death_date")

        self.required_if(
            CONSENT_WITHDRAWAL,
            field="offschedule_reason",
            field_required="consent_withdrawal_reason",
        )

        self.required_if(
            "included_in_error",
            field="offschedule_reason",
            field_required="included_in_error",
        )

        self.required_if(
            "included_in_error",
            field="offschedule_reason",
            field_required="included_in_error_date",
        )



from django.db import models
from edc_crf.crf_with_action_model_mixin import CrfWithActionModelMixin
from edc_model import models as edc_models
from edc_sites.models import SiteModelMixin
from edc_visit_tracking.model_mixins import SubjectVisitMissedModelMixin

from mocca_lists.models import SubjectVisitMissedReasons

from ..model_mixins import CrfModelMixin


class SubjectVisitMissed(
    SubjectVisitMissedModelMixin,
    CrfWithActionModelMixin,
    SiteModelMixin,
    edc_models.BaseUuidModel,
):

    action_identifier = models.CharField(max_length=50, null=True)

    tracking_identifier = models.CharField(max_length=30, null=True)

    missed_reasons = models.ManyToManyField(
        SubjectVisitMissedReasons, blank=True, related_name="+"
    )

    class Meta(
        CrfModelMixin.Meta,
        SubjectVisitMissedModelMixin.Meta,
        edc_models.BaseUuidModel.Meta,
    ):
        verbose_name = "Missed Visit Report"
        verbose_name_plural = "Missed Visit Report"



from edc_model_wrapper import ModelWrapper


class DeathReportModelWrapper(ModelWrapper):
    next_url_name = "subject_listboard_url"
    model = "mocca_ae.deathreport"
    next_url_attrs = ["subject_identifier"]

    @property
    def pk(self):
        return str(self.object.pk)

    @property
    def subject_identifier(self):
        return self.object.subject_identifier



from typing import Optional

from django import forms
from django.conf import settings
from edc_crf.forms import CrfFormValidatorMixin
from edc_crf.modelform_mixins import CrfModelFormMixin
from edc_dx import Diagnoses
from edc_dx.diagnoses import InitialReviewRequired
from edc_dx_review.utils import raise_if_clinical_review_does_not_exist
from edc_form_validators.form_validator import FormValidator
from edc_utils import convert_php_dateformat
from edc_visit_schedule.utils import raise_if_baseline

from ..models import Cd4Result


class ResultFormValidatorMixin(FormValidator):
    def validate_test_date_by_dx_date(
        self, prefix: str, dx_msg_label: str, test_date_fld: Optional[str] = None
    ) -> None:
        return self.validate_drawn_date_by_dx_date(
            prefix=prefix, dx_msg_label=dx_msg_label, drawn_date_fld=test_date_fld
        )

    def validate_drawn_date_by_dx_date(
        self, prefix: str, dx_msg_label: str, drawn_date_fld: Optional[str] = None
    ):
        drawn_date_fld = drawn_date_fld or "drawn_date"
        dx = Diagnoses(
            subject_visit=self.cleaned_data.get("subject_visit"),
            lte=True,
            limit_to_single_condition_prefix=prefix,
        )
        try:
            dx_date = dx.get_dx_date(prefix)
        except InitialReviewRequired:
            dx_date = None
        if not dx_date:
            raise forms.ValidationError(
                f"A {dx_msg_label} diagnosis has not been reported for this subject."
            )
        else:
            if dx_date > self.cleaned_data.get(drawn_date_fld):
                formatted_date = dx_date.strftime(
                    convert_php_dateformat(settings.SHORT_DATE_FORMAT)
                )
                raise forms.ValidationError(
                    {
                        "drawn_date": (
                            "Invalid. Subject was diagnosed with "
                            f"{dx_msg_label} on {formatted_date}."
                        )
                    }
                )


class Cd4ResultFormValidator(ResultFormValidatorMixin, CrfFormValidatorMixin, FormValidator):
    def clean(self):
        if self.cleaned_data.get("subject_visit"):
            raise_if_baseline(self.cleaned_data.get("subject_visit"))
            raise_if_clinical_review_does_not_exist(self.cleaned_data.get("subject_visit"))
        self.validate_drawn_date_by_dx_date("hiv", "HIV infection")


class Cd4ResultForm(CrfModelFormMixin, forms.ModelForm):
    form_validator_cls = Cd4ResultFormValidator

    class Meta:
        model = Cd4Result
        fields = "__all__"



from urllib.parse import parse_qs, urlsplit

from django.contrib import admin
from edc_constants.constants import YES
from edc_lab.admin import (
    RequisitionAdminMixin,
    requisition_fieldset,
    requisition_identifier_fieldset,
    requisition_status_fieldset,
    requisition_verify_fieldset,
)
from edc_model_admin import audit_fieldset_tuple

from ..admin_site import mocca_subject_admin
from ..forms import SubjectRequisitionForm
from ..models import SubjectRequisition
from .modeladmin_mixins import CrfModelAdmin


@admin.register(SubjectRequisition, site=mocca_subject_admin)
class SubjectRequisitionAdmin(RequisitionAdminMixin, CrfModelAdmin):

    # show_save_next = False

    form = SubjectRequisitionForm

    fieldsets = (
        (None, {"fields": ("subject_visit", "requisition_datetime", "panel")}),
        requisition_fieldset,
        requisition_status_fieldset,
        requisition_identifier_fieldset,
        requisition_verify_fieldset,
        audit_fieldset_tuple,
    )

    radio_fields = {
        "is_drawn": admin.VERTICAL,
        "reason_not_drawn": admin.VERTICAL,
        "item_type": admin.VERTICAL,
    }

    def get_search_results(self, request, queryset, search_term):
        queryset, use_distinct = super().get_search_results(request, queryset, search_term)
        path = urlsplit(request.META.get("HTTP_REFERER")).path
        query = urlsplit(request.META.get("HTTP_REFERER")).query
        if "bloodresult" in path:
            attrs = parse_qs(query)
            try:
                subject_visit = attrs.get("subject_visit")[0]
            except (TypeError, IndexError):
                pass
            else:
                queryset = queryset.filter(subject_visit=subject_visit, is_drawn=YES)
        return queryset, use_distinct



from edc_adherence.model_mixins import MedicationAdherenceModelMixin
from edc_model import models as edc_models

from ..model_mixins import CrfModelMixin


class DmMedicationAdherence(
    MedicationAdherenceModelMixin,
    CrfModelMixin,
    edc_models.BaseUuidModel,
):

    condition_label = "Diabetes"

    class Meta(CrfModelMixin.Meta, edc_models.BaseUuidModel.Meta):
        verbose_name = "Diabetes Medication Adherence"
        verbose_name_plural = "Diabetes Medication Adherence"



from edc_subject_dashboard.views import SubjectDashboardView


class DashboardView(SubjectDashboardView):

    consent_model = "mocca_consent.subjectconsent"
    navbar_selected_item = "consented_subject"
    visit_model = "mocca_subject.subjectvisit"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context.update({"recruitment_clinic_name": self.get_recruitment_clinic_name()})
        return context

    def get_recruitment_clinic_name(self):
        return self.consent.get_clinic_type_display()



# Generated by Django 3.2.9 on 2021-12-11 15:58

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('mocca_subject', '0024_auto_20211211_1856'),
    ]

    operations = [
        migrations.AlterField(
            model_name='bloodresultslipid',
            name='requisition',
            field=models.ForeignKey(blank=True, help_text='Start typing the requisition identifier or select one from this visit', null=True, on_delete=django.db.models.deletion.PROTECT, related_name='+', to='mocca_subject.subjectrequisition', verbose_name='Requisition'),
        ),
    ]



# Generated by Django 3.2.10 on 2021-12-09 17:43

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('mocca_prn', '0003_auto_20211209_2036'),
    ]

    operations = [
        migrations.AlterField(
            model_name='historicallosstofollowup',
            name='ltfu_category',
            field=models.CharField(choices=[('unknown_address', 'Changed to an unknown address'), ('never_returned', 'Did not return despite reminders'), ('bad_contact_details', 'Inaccurate contact details'), ('OTHER', 'Other, please specify ...')], max_length=25, verbose_name='Category of loss to follow up'),
        ),
        migrations.AlterField(
            model_name='losstofollowup',
            name='ltfu_category',
            field=models.CharField(choices=[('unknown_address', 'Changed to an unknown address'), ('never_returned', 'Did not return despite reminders'), ('bad_contact_details', 'Inaccurate contact details'), ('OTHER', 'Other, please specify ...')], max_length=25, verbose_name='Category of loss to follow up'),
        ),
    ]



from django import forms
from edc_action_item.forms.action_item_form_mixin import ActionItemFormMixin
from edc_constants.constants import NO, OTHER, YES
from edc_crf.forms import CrfFormValidatorMixin
from edc_crf.modelform_mixins import CrfModelFormMixin
from edc_dx_review.utils import (
    raise_if_both_ago_and_actual_date,
    raise_if_clinical_review_does_not_exist,
)
from edc_form_validators import FormValidator
from edc_model.utils import estimated_date_from_ago

from ..models import HivInitialReview


class HivInitialReviewFormValidator(
    CrfFormValidatorMixin,
    FormValidator,
):
    def clean(self):
        super().clean()
        raise_if_clinical_review_does_not_exist(self.cleaned_data.get("subject_visit"))

        raise_if_both_ago_and_actual_date(cleaned_data=self.cleaned_data)

        self.applicable_if(YES, field="receives_care", field_applicable="clinic")

        self.required_if(OTHER, field="clinic", field_required="clinic_other")

        self.required_if(YES, field="receives_care", field_required="arv_initiated")

        self.validate_art_initiation_date()

        self.required_if(YES, field="arv_initiated", field_required="has_vl")
        self.validate_viral_load()

        self.required_if(YES, field="arv_initiated", field_required="has_cd4")
        self.validate_cd4()

    def validate_art_initiation_date(self):
        self.not_required_if(
            NO,
            field="arv_initiated",
            field_required="arv_initiation_ago",
            inverse=False,
        )
        self.not_required_if(
            NO,
            field="arv_initiated",
            field_required="arv_initiation_actual_date",
            inverse=False,
        )
        if self.cleaned_data.get("art_initiated") == YES and not (
            self.cleaned_data.get("arv_initiation_ago")
            or self.cleaned_data.get("arv_initiation_actual_date")
        ):
            raise forms.ValidationError(
                {"arv_initiation_actual_date": "This field is required (or the above)."}
            )

        raise_if_both_ago_and_actual_date(
            dx_ago=self.cleaned_data.get("arv_initiation_ago"),
            dx_date=self.cleaned_data.get("arv_initiation_actual_date"),
        )

        if self.arv_initiation_date and self.dx_date:
            if self.arv_initiation_date < self.dx_date:
                field = self.which_field(
                    ago_field="arv_initiation_ago",
                    date_field="arv_initiation_actual_date",
                )
                raise forms.ValidationError(
                    {field: "Invalid. Cannot start ART before HIV diagnosis."}
                )

    def validate_viral_load(self):
        self.required_if(YES, field="has_vl", field_required="vl")
        self.required_if(YES, field="has_vl", field_required="vl_quantifier")
        self.required_if(YES, field="has_vl", field_required="vl_date")
        if self.cleaned_data.get("vl_date") and self.dx_date:
            if self.cleaned_data.get("vl_date") < self.dx_date:
                raise forms.ValidationError(
                    {"vl_date": "Invalid. Cannot be before HIV diagnosis."}
                )

    def validate_cd4(self):
        self.required_if(YES, field="has_cd4", field_required="cd4")
        self.required_if(YES, field="has_cd4", field_required="cd4_date")
        if self.cleaned_data.get("cd4_date") and self.dx_date:
            if self.cleaned_data.get("cd4_date") < self.dx_date:
                raise forms.ValidationError(
                    {"cd4_date": "Invalid. Cannot be before HIV diagnosis."}
                )

    @property
    def dx_date(self):
        if self.cleaned_data.get("dx_ago"):
            return estimated_date_from_ago(data=self.cleaned_data, ago_field="dx_ago")
        return self.cleaned_data.get("dx_date")

    @property
    def arv_initiation_date(self):
        if self.cleaned_data.get("arv_initiation_ago"):
            return estimated_date_from_ago(
                data=self.cleaned_data, ago_field="arv_initiation_ago"
            )
        return self.cleaned_data.get("arv_initiation_actual_date")

    def which_field(self, ago_field=None, date_field=None):
        if self.cleaned_data.get(ago_field):
            return ago_field
        if self.cleaned_data.get(date_field):
            return date_field
        return None


class HivInitialReviewForm(
    CrfModelFormMixin,
    ActionItemFormMixin,
    forms.ModelForm,
):
    form_validator_cls = HivInitialReviewFormValidator

    class Meta:
        model = HivInitialReview
        fields = "__all__"



from django.contrib import admin
from edc_model_admin import (
    ModelAdminFormAutoNumberMixin,
    ModelAdminFormInstructionsMixin,
    SimpleHistoryAdmin,
    TemplatesModelAdminMixin,
)

from ..admin_site import mocca_screening_admin
from ..forms import MoccaRegisterContactForm
from ..models import MoccaRegisterContact


@admin.register(MoccaRegisterContact, site=mocca_screening_admin)
class MoccaRegisterContactAdmin(
    TemplatesModelAdminMixin,
    ModelAdminFormAutoNumberMixin,
    ModelAdminFormInstructionsMixin,
    SimpleHistoryAdmin,
):
    form = MoccaRegisterContactForm
    show_object_tools = False



from django.conf import settings
from django.contrib.sites.models import Site
from django.core.validators import MinValueValidator
from django.db import models
from edc_model.models import BaseUuidModel
from edc_model.models.historical_records import HistoricalRecords
from edc_sites.models import CurrentSiteManager, SiteModelMixin
from edc_utils import convert_php_dateformat, get_utcnow

from ..choices import CLINIC_DAYS, SELECTION_METHOD


class DailyClosingLogManager(models.Manager):
    def get_by_natural_key(self, log_identifier):
        return self.get(log_identifier=log_identifier)


class DailyClosingLog(SiteModelMixin, BaseUuidModel):

    site = models.ForeignKey(
        Site,
        on_delete=models.PROTECT,
        null=True,
        related_name="+",
        blank=False,
    )

    log_date = models.DateField(verbose_name="Clinic date", default=get_utcnow)

    clinic_services = models.CharField(
        verbose_name="Which services are being offered at the clinic today?",
        max_length=25,
        choices=CLINIC_DAYS,
    )

    attended = models.IntegerField(
        verbose_name="Total number of patients who attended the clinic today",
        validators=[MinValueValidator(0)],
    )

    selection_method = models.CharField(
        verbose_name="How were patients selected to be approached?",
        max_length=25,
        choices=SELECTION_METHOD,
    )

    approached = models.IntegerField(
        verbose_name="Of those who attended, how many were approached by the study team",
        validators=[MinValueValidator(0)],
    )

    agreed_to_screen = models.IntegerField(
        verbose_name="Of those approached, how many agreed to be screened",
        validators=[MinValueValidator(0)],
    )

    comment = models.TextField(
        verbose_name="Additional Comments",
        null=True,
        blank=True,
    )

    on_site = CurrentSiteManager()

    objects = DailyClosingLogManager()

    history = HistoricalRecords()

    def __str__(self):
        return self.log_date.strftime(convert_php_dateformat(settings.DATE_FORMAT))

    def natural_key(self):
        return (self.log_date, self.site)

    class Meta:
        verbose_name = "Daily Closing Log"
        verbose_name_plural = "Daily Closing Logs"
        constraints = [
            models.UniqueConstraint(fields=["log_date", "site"], name="unique_date_for_site"),
        ]



from django.contrib import admin
from django_audit_fields.admin import audit_fieldset_tuple
from edc_crf.admin import crf_status_fieldset_tuple
from edc_form_label.form_label_modeladmin_mixin import FormLabelModelAdminMixin
from edc_model_admin import SimpleHistoryAdmin

from ..admin_site import mocca_subject_admin
from ..forms import ClinicalReviewForm
from ..models import ClinicalReview
from .modeladmin_mixins import CrfModelAdminMixin


@admin.register(ClinicalReview, site=mocca_subject_admin)
class ClinicalReviewAdmin(CrfModelAdminMixin, FormLabelModelAdminMixin, SimpleHistoryAdmin):
    form = ClinicalReviewForm

    fieldsets = (
        (None, {"fields": ("subject_visit", "report_datetime")}),
        (
            "HYPERTENSION",
            {
                "fields": (
                    "htn_test",
                    "htn_test_date",
                    "htn_reason",
                    "htn_reason_other",
                    "htn_dx",
                )
            },
        ),
        (
            "DIABETES",
            {
                "fields": (
                    "dm_test",
                    "dm_test_date",
                    "dm_reason",
                    "dm_reason_other",
                    "dm_dx",
                )
            },
        ),
        (
            "CHOLESTEROL",
            {
                "fields": (
                    "chol_test",
                    "chol_test_date",
                    "chol_reason",
                    "chol_reason_other",
                    "chol_dx",
                )
            },
        ),
        (
            "HIV",
            {
                "fields": (
                    "hiv_test",
                    "hiv_test_date",
                    "hiv_reason",
                    "hiv_reason_other",
                    "hiv_dx",
                )
            },
        ),
        ("Complications", {"fields": ("complications",)}),
        crf_status_fieldset_tuple,
        audit_fieldset_tuple,
    )

    radio_fields = {
        "chol_dx": admin.VERTICAL,
        "chol_test": admin.VERTICAL,
        "complications": admin.VERTICAL,
        "crf_status": admin.VERTICAL,
        "dm_dx": admin.VERTICAL,
        "dm_test": admin.VERTICAL,
        "hiv_dx": admin.VERTICAL,
        "hiv_test": admin.VERTICAL,
        "htn_dx": admin.VERTICAL,
        "htn_test": admin.VERTICAL,
    }

    filter_horizontal = ["htn_reason", "dm_reason", "hiv_reason", "chol_reason"]



# Generated by Django 3.0.9 on 2020-12-06 20:52

import django.db.models.deletion
import edc_model_fields.fields.other_charfield
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("mocca_screening", "0023_auto_20201206_2205"),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="icc",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                help_text="Either at this facility or elsewhere",
                max_length=25,
                verbose_name="Does the patient <u>currently</u> receive integrated care",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="icc_since_mocca",
            field=models.CharField(
                choices=[
                    ("no_interruption", "Yes, without interruption"),
                    ("some_interruption", "Yes, with some interruption"),
                    (
                        "No",
                        "No, not since completing follow up with the MOCCA (original) trial.",
                    ),
                    ("unknown", "Unknown"),
                ],
                max_length=25,
                verbose_name="Has the patient received integrated care since the leaving the MOCCA (orig) trial until now?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="icc_since_mocca_comment",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                null=True,
                verbose_name="If some interruption in integrated care, please explain",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="mocca_register",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="mocca_screening.MoccaRegister",
                verbose_name="MOCCA (original) register details",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="willing_to_consent",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                help_text="If Yes, begin the informed consent process.",
                max_length=25,
                verbose_name="Is the patient willing and able to participate in the `MOCCA extension` trial",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="icc",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                help_text="Either at this facility or elsewhere",
                max_length=25,
                verbose_name="Does the patient <u>currently</u> receive integrated care",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="icc_since_mocca",
            field=models.CharField(
                choices=[
                    ("no_interruption", "Yes, without interruption"),
                    ("some_interruption", "Yes, with some interruption"),
                    (
                        "No",
                        "No, not since completing follow up with the MOCCA (original) trial.",
                    ),
                    ("unknown", "Unknown"),
                ],
                max_length=25,
                verbose_name="Has the patient received integrated care since the leaving the MOCCA (orig) trial until now?",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="icc_since_mocca_comment",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                null=True,
                verbose_name="If some interruption in integrated care, please explain",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="mocca_register",
            field=models.OneToOneField(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                to="mocca_screening.MoccaRegister",
                verbose_name="MOCCA (original) register details",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="willing_to_consent",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                help_text="If Yes, begin the informed consent process.",
                max_length=25,
                verbose_name="Is the patient willing and able to participate in the `MOCCA extension` trial",
            ),
        ),
    ]



from django.conf import settings
from django.contrib import admin
from django.utils.safestring import mark_safe
from edc_adverse_event.modeladmin_mixins import AeInitialModelAdminMixin
from edc_model_admin import SimpleHistoryAdmin

from ..admin_site import mocca_ae_admin
from ..forms import AeInitialForm
from ..models import AeInitial


@admin.register(AeInitial, site=mocca_ae_admin)
class AeInitialAdmin(AeInitialModelAdminMixin, SimpleHistoryAdmin):

    form = AeInitialForm
    email_contact = settings.EMAIL_CONTACTS.get("ae_reports")
    additional_instructions = mark_safe(
        "Complete the initial AE report and forward to the TMG. "
        f'Email to <a href="mailto:{email_contact}">{email_contact}</a>'
    )



from .care_status import CareStatus
from .daily_closing_log import DailyClosingLog
from .mocca_register import MoccaRegister
from .mocca_register_contact import MoccaRegisterContact
from .model_mixins import CareModelMixin
from .signals import (
    mocca_register_contact_on_post_save,
    subject_refusal_on_post_delete,
    subject_refusal_on_post_save,
    subject_screening_on_post_save,
)
from .subject_refusal import SubjectRefusal
from .subject_refusal_screening import SubjectRefusalScreening
from .subject_screening import SubjectScreening



# Generated by Django 3.1.7 on 2021-03-26 18:07

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('mocca_subject', '0019_auto_20210326_2016'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='cholreview',
            name='care_start_date',
        ),
        migrations.RemoveField(
            model_name='cholreview',
            name='dx',
        ),
        migrations.RemoveField(
            model_name='cholreview',
            name='test_date',
        ),
        migrations.RemoveField(
            model_name='historicalcholreview',
            name='care_start_date',
        ),
        migrations.RemoveField(
            model_name='historicalcholreview',
            name='dx',
        ),
        migrations.RemoveField(
            model_name='historicalcholreview',
            name='test_date',
        ),
        migrations.AlterField(
            model_name='cholreview',
            name='managed_by',
            field=models.CharField(choices=[('drugs', 'Oral drugs'), ('diet_lifestyle', 'Diet and lifestyle alone')], default='N/A', max_length=25, verbose_name="How will the patient's High Cholesterol be managed going forward?"),
        ),
        migrations.AlterField(
            model_name='historicalcholreview',
            name='managed_by',
            field=models.CharField(choices=[('drugs', 'Oral drugs'), ('diet_lifestyle', 'Diet and lifestyle alone')], default='N/A', max_length=25, verbose_name="How will the patient's High Cholesterol be managed going forward?"),
        ),
    ]



from django.contrib import admin
from django_audit_fields.admin import audit_fieldset_tuple
from edc_model_admin import (
    ModelAdminFormAutoNumberMixin,
    ModelAdminFormInstructionsMixin,
    ModelAdminNextUrlRedirectMixin,
    TemplatesModelAdminMixin,
)
from edc_model_admin.model_admin_simple_history import SimpleHistoryAdmin

from ..admin_site import mocca_screening_admin
from ..forms import SubjectRefusalScreeningForm
from ..models import SubjectRefusalScreening


@admin.register(SubjectRefusalScreening, site=mocca_screening_admin)
class SubjectRefusalScreeningAdmin(
    TemplatesModelAdminMixin,
    ModelAdminFormAutoNumberMixin,
    ModelAdminFormInstructionsMixin,
    ModelAdminNextUrlRedirectMixin,
    SimpleHistoryAdmin,
):
    form = SubjectRefusalScreeningForm

    fieldsets = (
        [
            None,
            {
                "fields": (
                    "mocca_register",
                    "report_datetime",
                    "reason",
                    "other_reason",
                    "comment",
                )
            },
        ],
        audit_fieldset_tuple,
    )

    list_display = (
        "mocca_register",
        "report_datetime",
        "reason",
        "user_created",
        "created",
    )

    list_filter = ("report_datetime", "reason")

    search_fields = (
        "mocca_register__mocca_study_identifier",
        "mocca_register__initials",
        "mocca_register__mocca_screening_identifier",
    )

    radio_fields = {"reason": admin.VERTICAL}

    def formfield_for_foreignkey(self, db_field, request, **kwargs):
        if db_field.name == "mocca_register" and request.GET.get("mocca_register"):
            kwargs["queryset"] = db_field.related_model.objects.filter(
                pk=request.GET.get("mocca_register", 0)
            )
        return super().formfield_for_foreignkey(db_field, request, **kwargs)



# Generated by Django 3.0.9 on 2021-01-14 19:57

import uuid

import _socket
import django.contrib.sites.managers
import django.db.models.deletion
import django_audit_fields.fields.hostname_modification_field
import django_audit_fields.fields.userfield
import django_audit_fields.fields.uuid_auto_field
import django_audit_fields.models.audit_model_mixin
import django_revision.revision_field
import edc_model.models.validators.date
import edc_protocol.validators
import edc_utils.date
import edc_visit_tracking.managers
import simple_history.models
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("sites", "0002_alter_domain_unique"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("mocca_subject", "0004_remove_reasonforvisit_health_services"),
    ]

    operations = [
        migrations.CreateModel(
            name="PatientHealth",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                ("device_created", models.CharField(blank=True, max_length=10)),
                ("device_modified", models.CharField(blank=True, max_length=10)),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                (
                    "report_datetime",
                    models.DateTimeField(
                        default=edc_utils.date.get_utcnow,
                        help_text="If reporting today, use today's date/time, otherwise use the date/time this information was reported.",
                        validators=[
                            edc_protocol.validators.datetime_not_before_study_start,
                            edc_model.models.validators.date.datetime_not_future,
                        ],
                        verbose_name="Report Date",
                    ),
                ),
                (
                    "consent_model",
                    models.CharField(editable=False, max_length=50, null=True),
                ),
                (
                    "consent_version",
                    models.CharField(editable=False, max_length=10, null=True),
                ),
                (
                    "crf_status",
                    models.CharField(
                        choices=[
                            ("INCOMPLETE", "Incomplete (some data pending)"),
                            ("COMPLETE", "Complete"),
                        ],
                        default="INCOMPLETE",
                        help_text="If some data is still pending, flag this CRF as incomplete",
                        max_length=25,
                        verbose_name="CRF status",
                    ),
                ),
                (
                    "crf_status_comments",
                    models.TextField(
                        blank=True,
                        help_text="for example, why some data is still pending",
                        null=True,
                        verbose_name="Any comments related to status of this CRF",
                    ),
                ),
                (
                    "ph9interst",
                    models.CharField(
                        choices=[
                            ("not_at_all", "Not at all"),
                            ("several_days", "Several days"),
                            ("more_than_half", "More than half the days"),
                            ("nearly_everyday", "Nearly everyday"),
                        ],
                        max_length=15,
                        verbose_name="Little interest or pleasure in doing things",
                    ),
                ),
                (
                    "ph9feel",
                    models.CharField(
                        choices=[
                            ("not_at_all", "Not at all"),
                            ("several_days", "Several days"),
                            ("more_than_half", "More than half the days"),
                            ("nearly_everyday", "Nearly everyday"),
                        ],
                        max_length=15,
                        verbose_name="Feeling down, depressed or hopeless",
                    ),
                ),
                (
                    "ph9troubl",
                    models.CharField(
                        choices=[
                            ("not_at_all", "Not at all"),
                            ("several_days", "Several days"),
                            ("more_than_half", "More than half the days"),
                            ("nearly_everyday", "Nearly everyday"),
                        ],
                        max_length=15,
                        verbose_name="Trouble falling/staying asleep, sleeping too much",
                    ),
                ),
                (
                    "ph9tired",
                    models.CharField(
                        choices=[
                            ("not_at_all", "Not at all"),
                            ("several_days", "Several days"),
                            ("more_than_half", "More than half the days"),
                            ("nearly_everyday", "Nearly everyday"),
                        ],
                        max_length=15,
                        verbose_name="Feeling tired or having little energy",
                    ),
                ),
                (
                    "ph9appetit",
                    models.CharField(
                        choices=[
                            ("not_at_all", "Not at all"),
                            ("several_days", "Several days"),
                            ("more_than_half", "More than half the days"),
                            ("nearly_everyday", "Nearly everyday"),
                        ],
                        max_length=15,
                        verbose_name="Poor appetite or over eating",
                    ),
                ),
                (
                    "ph9badabt",
                    models.CharField(
                        choices=[
                            ("not_at_all", "Not at all"),
                            ("several_days", "Several days"),
                            ("more_than_half", "More than half the days"),
                            ("nearly_everyday", "Nearly everyday"),
                        ],
                        max_length=15,
                        verbose_name="Feeling bad about yourself or that you are a failure or have let yourself or your family down",
                    ),
                ),
                (
                    "ph9concen",
                    models.CharField(
                        choices=[
                            ("not_at_all", "Not at all"),
                            ("several_days", "Several days"),
                            ("more_than_half", "More than half the days"),
                            ("nearly_everyday", "Nearly everyday"),
                        ],
                        max_length=15,
                        verbose_name="Trouble concentrating on things such as reading the newspapers or watching television",
                    ),
                ),
                (
                    "ph9moving",
                    models.CharField(
                        choices=[
                            ("not_at_all", "Not at all"),
                            ("several_days", "Several days"),
                            ("more_than_half", "More than half the days"),
                            ("nearly_everyday", "Nearly everyday"),
                        ],
                        max_length=15,
                        verbose_name="Moving or speaking so slowly  that other people could have noticed or the opposite: being so fidgety or restless that you have been moving around a lot more than usual",
                    ),
                ),
                (
                    "phpthough",
                    models.CharField(
                        choices=[
                            ("not_at_all", "Not at all"),
                            ("several_days", "Several days"),
                            ("more_than_half", "More than half the days"),
                            ("nearly_everyday", "Nearly everyday"),
                        ],
                        max_length=15,
                        verbose_name="Thoughts that you would be better off dead or of hurting yourself in some way",
                    ),
                ),
                (
                    "ph9functio",
                    models.CharField(
                        choices=[
                            ("not_at_all", "Not at all"),
                            ("several_days", "Several days"),
                            ("more_than_half", "More than half the days"),
                            ("nearly_everyday", "Nearly everyday"),
                        ],
                        max_length=15,
                        verbose_name="If you checked off any problems on this questionnaire so far, how difficult have these problems made it for you to do your work, take care of things at home or get along with other people?",
                    ),
                ),
                (
                    "site",
                    models.ForeignKey(
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="+",
                        to="sites.Site",
                    ),
                ),
                (
                    "subject_visit",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.PROTECT,
                        to="mocca_subject.SubjectVisit",
                    ),
                ),
            ],
            options={
                "verbose_name": "Patient Health Questionnaire-9 (PHQ9-)",
                "verbose_name_plural": "Patient Health Questionnaires-9 (PHQ9-)",
                "ordering": ("-modified", "-created"),
                "get_latest_by": "modified",
                "abstract": False,
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
            },
            managers=[
                ("on_site", django.contrib.sites.managers.CurrentSiteManager()),
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
            ],
        ),
        migrations.CreateModel(
            name="HistoricalPatientHealth",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                ("device_created", models.CharField(blank=True, max_length=10)),
                ("device_modified", models.CharField(blank=True, max_length=10)),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        db_index=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                    ),
                ),
                (
                    "report_datetime",
                    models.DateTimeField(
                        default=edc_utils.date.get_utcnow,
                        help_text="If reporting today, use today's date/time, otherwise use the date/time this information was reported.",
                        validators=[
                            edc_protocol.validators.datetime_not_before_study_start,
                            edc_model.models.validators.date.datetime_not_future,
                        ],
                        verbose_name="Report Date",
                    ),
                ),
                (
                    "consent_model",
                    models.CharField(editable=False, max_length=50, null=True),
                ),
                (
                    "consent_version",
                    models.CharField(editable=False, max_length=10, null=True),
                ),
                (
                    "crf_status",
                    models.CharField(
                        choices=[
                            ("INCOMPLETE", "Incomplete (some data pending)"),
                            ("COMPLETE", "Complete"),
                        ],
                        default="INCOMPLETE",
                        help_text="If some data is still pending, flag this CRF as incomplete",
                        max_length=25,
                        verbose_name="CRF status",
                    ),
                ),
                (
                    "crf_status_comments",
                    models.TextField(
                        blank=True,
                        help_text="for example, why some data is still pending",
                        null=True,
                        verbose_name="Any comments related to status of this CRF",
                    ),
                ),
                (
                    "history_id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                (
                    "ph9interst",
                    models.CharField(
                        choices=[
                            ("not_at_all", "Not at all"),
                            ("several_days", "Several days"),
                            ("more_than_half", "More than half the days"),
                            ("nearly_everyday", "Nearly everyday"),
                        ],
                        max_length=15,
                        verbose_name="Little interest or pleasure in doing things",
                    ),
                ),
                (
                    "ph9feel",
                    models.CharField(
                        choices=[
                            ("not_at_all", "Not at all"),
                            ("several_days", "Several days"),
                            ("more_than_half", "More than half the days"),
                            ("nearly_everyday", "Nearly everyday"),
                        ],
                        max_length=15,
                        verbose_name="Feeling down, depressed or hopeless",
                    ),
                ),
                (
                    "ph9troubl",
                    models.CharField(
                        choices=[
                            ("not_at_all", "Not at all"),
                            ("several_days", "Several days"),
                            ("more_than_half", "More than half the days"),
                            ("nearly_everyday", "Nearly everyday"),
                        ],
                        max_length=15,
                        verbose_name="Trouble falling/staying asleep, sleeping too much",
                    ),
                ),
                (
                    "ph9tired",
                    models.CharField(
                        choices=[
                            ("not_at_all", "Not at all"),
                            ("several_days", "Several days"),
                            ("more_than_half", "More than half the days"),
                            ("nearly_everyday", "Nearly everyday"),
                        ],
                        max_length=15,
                        verbose_name="Feeling tired or having little energy",
                    ),
                ),
                (
                    "ph9appetit",
                    models.CharField(
                        choices=[
                            ("not_at_all", "Not at all"),
                            ("several_days", "Several days"),
                            ("more_than_half", "More than half the days"),
                            ("nearly_everyday", "Nearly everyday"),
                        ],
                        max_length=15,
                        verbose_name="Poor appetite or over eating",
                    ),
                ),
                (
                    "ph9badabt",
                    models.CharField(
                        choices=[
                            ("not_at_all", "Not at all"),
                            ("several_days", "Several days"),
                            ("more_than_half", "More than half the days"),
                            ("nearly_everyday", "Nearly everyday"),
                        ],
                        max_length=15,
                        verbose_name="Feeling bad about yourself or that you are a failure or have let yourself or your family down",
                    ),
                ),
                (
                    "ph9concen",
                    models.CharField(
                        choices=[
                            ("not_at_all", "Not at all"),
                            ("several_days", "Several days"),
                            ("more_than_half", "More than half the days"),
                            ("nearly_everyday", "Nearly everyday"),
                        ],
                        max_length=15,
                        verbose_name="Trouble concentrating on things such as reading the newspapers or watching television",
                    ),
                ),
                (
                    "ph9moving",
                    models.CharField(
                        choices=[
                            ("not_at_all", "Not at all"),
                            ("several_days", "Several days"),
                            ("more_than_half", "More than half the days"),
                            ("nearly_everyday", "Nearly everyday"),
                        ],
                        max_length=15,
                        verbose_name="Moving or speaking so slowly  that other people could have noticed or the opposite: being so fidgety or restless that you have been moving around a lot more than usual",
                    ),
                ),
                (
                    "phpthough",
                    models.CharField(
                        choices=[
                            ("not_at_all", "Not at all"),
                            ("several_days", "Several days"),
                            ("more_than_half", "More than half the days"),
                            ("nearly_everyday", "Nearly everyday"),
                        ],
                        max_length=15,
                        verbose_name="Thoughts that you would be better off dead or of hurting yourself in some way",
                    ),
                ),
                (
                    "ph9functio",
                    models.CharField(
                        choices=[
                            ("not_at_all", "Not at all"),
                            ("several_days", "Several days"),
                            ("more_than_half", "More than half the days"),
                            ("nearly_everyday", "Nearly everyday"),
                        ],
                        max_length=15,
                        verbose_name="If you checked off any problems on this questionnaire so far, how difficult have these problems made it for you to do your work, take care of things at home or get along with other people?",
                    ),
                ),
                ("history_date", models.DateTimeField()),
                ("history_change_reason", models.CharField(max_length=100, null=True)),
                (
                    "history_type",
                    models.CharField(
                        choices=[("+", "Created"), ("~", "Changed"), ("-", "Deleted")],
                        max_length=1,
                    ),
                ),
                (
                    "history_user",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="+",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "site",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="sites.Site",
                    ),
                ),
                (
                    "subject_visit",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="mocca_subject.SubjectVisit",
                    ),
                ),
            ],
            options={
                "verbose_name": "historical Patient Health Questionnaire-9 (PHQ9-)",
                "ordering": ("-history_date", "-history_id"),
                "get_latest_by": "history_date",
            },
            bases=(simple_history.models.HistoricalChanges, models.Model),
        ),
        migrations.AddIndex(
            model_name="patienthealth",
            index=models.Index(
                fields=["subject_visit", "site", "id"],
                name="mocca_subje_subject_e0c51a_idx",
            ),
        ),
    ]



from django import forms
from edc_crf.forms import CrfFormValidatorMixin
from edc_crf.modelform_mixins import CrfModelFormMixin
from edc_form_validators.form_validator import FormValidator

from ..models import DrugRefillHtn
from .drug_refill_mixins import DrugRefillFormValidatorMixin


class DrugRefillHtnFormValidator(
    DrugRefillFormValidatorMixin, CrfFormValidatorMixin, FormValidator
):
    def clean(self) -> None:
        self.drug_refill_clean()


class DrugRefillHtnForm(CrfModelFormMixin, forms.ModelForm):
    form_validator_cls = DrugRefillHtnFormValidator

    class Meta:
        model = DrugRefillHtn
        fields = "__all__"



# Generated by Django 3.1.7 on 2021-03-31 14:28

from django.db import migrations, models
import django.db.models.deletion
import edc_model.models.validators.date


class Migration(migrations.Migration):

    dependencies = [
        ('mocca_subject', '0020_auto_20210326_2107'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='glucose',
            options={'default_permissions': ('add', 'change', 'delete', 'view', 'export', 'import'), 'get_latest_by': 'modified', 'ordering': ('-modified', '-created'), 'verbose_name': 'Glucose: Followup', 'verbose_name_plural': 'Glucose: Followup'},
        ),
        migrations.AlterModelOptions(
            name='historicalglucose',
            options={'get_latest_by': 'history_date', 'ordering': ('-history_date', '-history_id'), 'verbose_name': 'historical Glucose: Followup'},
        ),
        migrations.RemoveField(
            model_name='dmreview',
            name='care_start_date',
        ),
        migrations.RemoveField(
            model_name='dmreview',
            name='dx',
        ),
        migrations.RemoveField(
            model_name='dmreview',
            name='glucose',
        ),
        migrations.RemoveField(
            model_name='dmreview',
            name='glucose_date',
        ),
        migrations.RemoveField(
            model_name='dmreview',
            name='glucose_fasted',
        ),
        migrations.RemoveField(
            model_name='dmreview',
            name='glucose_quantifier',
        ),
        migrations.RemoveField(
            model_name='dmreview',
            name='glucose_units',
        ),
        migrations.RemoveField(
            model_name='dmreview',
            name='test_date',
        ),
        migrations.RemoveField(
            model_name='historicaldmreview',
            name='care_start_date',
        ),
        migrations.RemoveField(
            model_name='historicaldmreview',
            name='dx',
        ),
        migrations.RemoveField(
            model_name='historicaldmreview',
            name='glucose',
        ),
        migrations.RemoveField(
            model_name='historicaldmreview',
            name='glucose_date',
        ),
        migrations.RemoveField(
            model_name='historicaldmreview',
            name='glucose_fasted',
        ),
        migrations.RemoveField(
            model_name='historicaldmreview',
            name='glucose_quantifier',
        ),
        migrations.RemoveField(
            model_name='historicaldmreview',
            name='glucose_units',
        ),
        migrations.RemoveField(
            model_name='historicaldmreview',
            name='test_date',
        ),
        migrations.AddField(
            model_name='clinicalreviewbaseline',
            name='complications',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No')], default='-', help_text='If Yes, complete the `Complications` CRF', max_length=15, verbose_name='Since last seen, has the patient had any complications'),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='glucose',
            name='fasting',
            field=models.CharField(blank=True, choices=[('fasting', 'Fasting'), ('non_fasting', 'Non-fasting')], max_length=25, null=True, verbose_name='Was this fasting or non-fasting?'),
        ),
        migrations.AddField(
            model_name='glucose',
            name='glucose_abnormal',
            field=models.CharField(blank=True, choices=[('Yes', 'Yes'), ('No', 'No')], max_length=25, null=True, verbose_name='abnormal'),
        ),
        migrations.AddField(
            model_name='glucose',
            name='glucose_assay_datetime',
            field=models.DateTimeField(blank=True, null=True, validators=[edc_model.models.validators.date.datetime_not_future], verbose_name='Result Report Date and Time'),
        ),
        migrations.AddField(
            model_name='glucose',
            name='glucose_reportable',
            field=models.CharField(blank=True, choices=[('N/A', 'Not applicable'), ('3', 'Yes, grade 3'), ('4', 'Yes, grade 4'), ('No', 'Not reportable'), ('Already reported', 'Already reported'), ('present_at_baseline', 'Present at baseline')], max_length=25, null=True, verbose_name='reportable'),
        ),
        migrations.AddField(
            model_name='glucose',
            name='glucose_requisition',
            field=models.ForeignKey(blank=True, help_text='Start typing the requisition identifier or select one from this visit', null=True, on_delete=django.db.models.deletion.PROTECT, related_name='bg', to='mocca_subject.subjectrequisition', verbose_name='Requisition'),
        ),
        migrations.AddField(
            model_name='glucose',
            name='is_poc',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No')], max_length=15, null=True, verbose_name='Was a point-of-care test used?'),
        ),
        migrations.AddField(
            model_name='historicalclinicalreviewbaseline',
            name='complications',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No')], default='-', help_text='If Yes, complete the `Complications` CRF', max_length=15, verbose_name='Since last seen, has the patient had any complications'),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='historicalglucose',
            name='fasting',
            field=models.CharField(blank=True, choices=[('fasting', 'Fasting'), ('non_fasting', 'Non-fasting')], max_length=25, null=True, verbose_name='Was this fasting or non-fasting?'),
        ),
        migrations.AddField(
            model_name='historicalglucose',
            name='glucose_abnormal',
            field=models.CharField(blank=True, choices=[('Yes', 'Yes'), ('No', 'No')], max_length=25, null=True, verbose_name='abnormal'),
        ),
        migrations.AddField(
            model_name='historicalglucose',
            name='glucose_assay_datetime',
            field=models.DateTimeField(blank=True, null=True, validators=[edc_model.models.validators.date.datetime_not_future], verbose_name='Result Report Date and Time'),
        ),
        migrations.AddField(
            model_name='historicalglucose',
            name='glucose_reportable',
            field=models.CharField(blank=True, choices=[('N/A', 'Not applicable'), ('3', 'Yes, grade 3'), ('4', 'Yes, grade 4'), ('No', 'Not reportable'), ('Already reported', 'Already reported'), ('present_at_baseline', 'Present at baseline')], max_length=25, null=True, verbose_name='reportable'),
        ),
        migrations.AddField(
            model_name='historicalglucose',
            name='glucose_requisition',
            field=models.ForeignKey(blank=True, db_constraint=False, help_text='Start typing the requisition identifier or select one from this visit', null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='mocca_subject.subjectrequisition', verbose_name='Requisition'),
        ),
        migrations.AddField(
            model_name='historicalglucose',
            name='is_poc',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No')], max_length=15, null=True, verbose_name='Was a point-of-care test used?'),
        ),
        migrations.AddField(
            model_name='historicalglucosebaseline',
            name='fasting',
            field=models.CharField(blank=True, choices=[('fasting', 'Fasting'), ('non_fasting', 'Non-fasting')], max_length=25, null=True, verbose_name='Was this fasting or non-fasting?'),
        ),
        migrations.AddField(
            model_name='historicalglucosebaseline',
            name='glucose_abnormal',
            field=models.CharField(blank=True, choices=[('Yes', 'Yes'), ('No', 'No')], max_length=25, null=True, verbose_name='abnormal'),
        ),
        migrations.AddField(
            model_name='historicalglucosebaseline',
            name='glucose_assay_datetime',
            field=models.DateTimeField(blank=True, null=True, validators=[edc_model.models.validators.date.datetime_not_future], verbose_name='Result Report Date and Time'),
        ),
        migrations.AddField(
            model_name='historicalglucosebaseline',
            name='glucose_reportable',
            field=models.CharField(blank=True, choices=[('N/A', 'Not applicable'), ('3', 'Yes, grade 3'), ('4', 'Yes, grade 4'), ('No', 'Not reportable'), ('Already reported', 'Already reported'), ('present_at_baseline', 'Present at baseline')], max_length=25, null=True, verbose_name='reportable'),
        ),
        migrations.AddField(
            model_name='historicalglucosebaseline',
            name='glucose_requisition',
            field=models.ForeignKey(blank=True, db_constraint=False, help_text='Start typing the requisition identifier or select one from this visit', null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='mocca_subject.subjectrequisition', verbose_name='Requisition'),
        ),
        migrations.AddField(
            model_name='historicalglucosebaseline',
            name='is_poc',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No')], max_length=15, null=True, verbose_name='Was a point-of-care test used?'),
        ),
    ]



# Generated by Django 3.1.5 on 2021-01-31 17:37

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("mocca_subject", "0007_auto_20210115_0423"),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicalsubjectvisit",
            name="info_source",
            field=models.CharField(
                choices=[
                    ("patient", "Patient"),
                    ("patient_representative", "Patient Representative"),
                    (
                        "patient_and_outpatient",
                        "Patient, hospital notes and/or outpatient card",
                    ),
                    ("collateral_history", "Collateral History from relative/guardian"),
                    ("hospital_notes", "Hospital notes"),
                    ("outpatient_cards", "Outpatient cards"),
                    ("N/A", "Not applicable"),
                    ("OTHER", "Other"),
                ],
                max_length=25,
                verbose_name="What is the main source of this information?",
            ),
        ),
        migrations.AlterField(
            model_name="subjectvisit",
            name="info_source",
            field=models.CharField(
                choices=[
                    ("patient", "Patient"),
                    ("patient_representative", "Patient Representative"),
                    (
                        "patient_and_outpatient",
                        "Patient, hospital notes and/or outpatient card",
                    ),
                    ("collateral_history", "Collateral History from relative/guardian"),
                    ("hospital_notes", "Hospital notes"),
                    ("outpatient_cards", "Outpatient cards"),
                    ("N/A", "Not applicable"),
                    ("OTHER", "Other"),
                ],
                max_length=25,
                verbose_name="What is the main source of this information?",
            ),
        ),
    ]



DIET_LIFESTYLE = "diet_lifestyle"
DRUGS = "drugs"
GTE_3HRS = "gte_3hr"
HIV = "hiv"  # TODO: reconcile with edc_constants.HIV
INSULIN = "insulin"
INTEGRATED = "integrated"
NCD = "ncd"
SITTING = "sitting"
THIS_CLINIC = "this_clinic"
VERTICAL = "vertical"



from django import forms
from edc_adverse_event.forms import AeFollowupModelFormMixin

from ..models import AeFollowup


class AeFollowupForm(AeFollowupModelFormMixin, forms.ModelForm):
    class Meta:
        model = AeFollowup
        fields = "__all__"



from edc_lab import RequisitionPanel
from edc_lab_panel.processing_profiles import lipids_processing

lipids_panel = RequisitionPanel(
    name="chemistry",
    verbose_name="Chemistry: Lipids",
    abbreviation="LIPIDS",
    processing_profile=lipids_processing,
    utest_ids=["ldl", "hdl", "trig", "chol"],
    reference_range_collection_name="mocca",
)



from .blood_results_lipid_form import (
    BloodResultsLipidForm,
    BloodResultsLipidFormValidator,
)
from .cd4_result_form import Cd4ResultForm
from .chol_initial_review_form import CholInitialReviewForm
from .chol_medication_adherence_form import CholMedicationAdherenceForm
from .chol_review_form import CholReviewForm
from .clinical_review_baseline_form import ClinicalReviewBaselineForm
from .clinical_review_form import ClinicalReviewForm
from .complications_baseline_form import ComplicationsBaselineForm
from .complications_followup_form import ComplicationsFollowupForm
from .dm_initial_review_form import DmInitialReviewForm
from .dm_medication_adherence_form import DmMedicationAdherenceForm
from .dm_review_form import DmReviewForm
from .drug_refill_chol_form import DrugRefillCholForm
from .drug_refill_dm_form import DrugRefillDmForm
from .drug_refill_hiv_form import DrugRefillHivForm
from .drug_refill_htn_form import DrugRefillHtnForm
from .family_history_form import FamilyHistoryForm
from .glucose_baseline_form import GlucoseBaselineForm
from .glucose_followup_form import GlucoseFollowupForm
from .hiv_initial_review_form import HivInitialReviewForm
from .hiv_medication_adherence_form import HivMedicationAdherenceForm
from .hiv_review_form import HivReviewForm
from .htn_initial_review_form import HtnInitialReviewForm
from .htn_medication_adherence_form import HtnMedicationAdherenceForm
from .htn_review_form import HtnReviewForm
from .indicators_form import IndicatorsForm
from .medications_form import MedicationsForm
from .next_appointment_form import NextAppointmentForm
from .other_baseline_data_form import OtherBaselineDataForm
from .patient_health_form import PatientHealthForm
from .subject_requisition_form import SubjectRequisitionForm
from .subject_visit_form import SubjectVisitForm
from .subject_visit_missed_form import SubjectVisitMissedForm
from .viral_load_result_form import ViralLoadResultForm



from edc_blood_results.form_validator_mixins import BloodResultsGluFormValidatorMixin
from edc_form_validators import FormValidator
from edc_lab_panel.panels import blood_glucose_panel, hba1c_panel

from mocca_labs.panels import lipids_panel

from .blood_results_form_validator_mixin import BloodResultsFormValidatorMixin


class BloodResultsGluFormValidator(
    BloodResultsGluFormValidatorMixin, BloodResultsFormValidatorMixin, FormValidator
):
    panel = blood_glucose_panel
    reportable_grades = []


class BloodResultsHba1cFormValidator(BloodResultsFormValidatorMixin, FormValidator):
    panels = hba1c_panel
    reportable_grades = []


class BloodResultsLipidFormValidator(BloodResultsFormValidatorMixin, FormValidator):
    panel = lipids_panel
    reportable_grades = []


# class BloodResultsGluFormValidator(BloodResultsFormValidatorMixin):
#
#     requisition_field = "glucose_requisition"
#     assay_datetime_field = "glucose_assay_datetime"
#     field_names = ["glucose", "fasting"]
#     panels = [blood_glucose_panel]
#     poc_panels = [blood_glucose_poc_panel]
#
#     @property
#     def extra_options(self):
#         if not self.cleaned_data.get("fasting"):
#             raise forms.ValidationError({"fasting": "This field is required."})
#         fasting = True if self.cleaned_data.get("fasting") == FASTING else False
#         return dict(fasting=fasting)


# class BloodResultsHba1cFormValidator(BloodResultsFormValidatorMixin):
#
#     requisition_field = "hba1c_requisition"
#     assay_datetime_field = "hba1c_assay_datetime"
#     field_names = ["hba1c"]
#     panels = [hba1c_panel]
#     poc_panels = [hba1c_poc_panel]
#
#     def validate_reportable_fields(self, **kwargs):
#         pass

#
# class BloodResultsLipidFormValidator(BloodResultsFormValidatorMixin):
#
#     requisition_field = "lipid_requisition"
#     assay_datetime_field = "lipid_assay_datetime"
#     field_names = ["ldl", "hdl", "trig", "chol"]
#     panels = [chemistry_lipids_panel]
#     reportable_grades = []



from django.test import TestCase
from edc_appointment.models import Appointment
from edc_model_wrapper.tests import ModelWrapperTestHelper
from edc_subject_model_wrappers import (
    AppointmentModelWrapper,
    SubjectConsentModelWrapper,
    SubjectLocatorModelWrapper,
    SubjectVisitModelWrapper,
)

from mocca_consent.models import SubjectConsent
from mocca_dashboard.model_wrappers import SubjectScreeningModelWrapper
from mocca_screening.models import SubjectScreening
from mocca_subject.models import SubjectVisit

from ..mocca_test_case_mixin import MoccaTestCaseMixin


class SubjectModelWrapperTestHelper(ModelWrapperTestHelper):
    dashboard_url = "/subject_dashboard/"


class ScreeningModelWrapperTestHelper(ModelWrapperTestHelper):
    dashboard_url = "/screening_listboard/"


class TestModelWrappers(MoccaTestCaseMixin, TestCase):
    model_wrapper_helper_cls = SubjectModelWrapperTestHelper

    def setUp(self):
        subject_screening = self.get_subject_screening()
        subject_consent = self.get_subject_consent(subject_screening)
        self.subject_identifier = subject_consent.subject_identifier

    def test_subject_screening(self):
        subject_screening = SubjectScreening.objects.all()[0]
        helper = ScreeningModelWrapperTestHelper(
            model_wrapper=SubjectScreeningModelWrapper, model_obj=subject_screening
        )
        helper.test(self)

    def test_subject_consent(self):
        subject_consent = SubjectConsent.objects.all()[0]
        helper = ModelWrapperTestHelper(
            model_wrapper=SubjectConsentModelWrapper, model_obj=subject_consent
        )
        helper.test(self)

    def test_subject_locator(self):
        helper = ModelWrapperTestHelper(
            model_wrapper=SubjectLocatorModelWrapper,
            subject_identifier=self.subject_identifier,
        )
        helper.test(self)

    def test_appointment(self):
        appointment = Appointment.objects.all()[0]
        helper = ModelWrapperTestHelper(
            model_wrapper=AppointmentModelWrapper, model_obj=appointment
        )
        helper.test(self)

    def test_subject_visit(self):
        appointment = Appointment.objects.all()[0]
        subject_visit = SubjectVisit.objects.create(
            appointment=appointment, reason="scheduled"
        )
        helper = ModelWrapperTestHelper(
            model_wrapper=SubjectVisitModelWrapper, model_obj=subject_visit
        )
        helper.test(self)



from django.db import models
from edc_action_item.models import ActionModelMixin
from edc_constants.choices import YES_NO_NA
from edc_constants.constants import NOT_APPLICABLE
from edc_identifier.model_mixins import TrackingModelMixin
from edc_model import models as edc_models
from edc_offstudy.constants import END_OF_STUDY_ACTION
from edc_sites.models import CurrentSiteManager
from edc_visit_schedule.model_mixins import (
    OffScheduleModelMixin,
    VisitScheduleFieldsModelMixin,
)

from mocca_lists.models import OffstudyReasons


class EndOfStudy(
    OffScheduleModelMixin,
    VisitScheduleFieldsModelMixin,
    ActionModelMixin,
    TrackingModelMixin,
    edc_models.BaseUuidModel,
):

    action_name = END_OF_STUDY_ACTION

    tracking_identifier_prefix = "ST"

    offschedule_datetime = models.DateTimeField(
        verbose_name="Date patient was terminated from the study",
        validators=[edc_models.datetime_not_future],
        blank=False,
        null=True,
    )

    last_study_fu_date = models.DateField(
        verbose_name="Date of last research follow up (if different):",
        validators=[edc_models.date_not_future],
        blank=True,
        null=True,
    )

    offschedule_reason = models.ForeignKey(
        OffstudyReasons,
        verbose_name="Reason patient was terminated from the study",
        on_delete=models.PROTECT,
        null=True,
    )

    other_offschedule_reason = models.TextField(
        verbose_name="If OTHER, please specify", max_length=500, blank=True, null=True
    )

    ltfu_last_alive_date = models.DateField(
        verbose_name="If lost to followup, date last known to be alive",
        validators=[edc_models.date_not_future],
        blank=True,
        null=True,
    )

    death_date = models.DateField(
        verbose_name="If deceased, date of death",
        validators=[edc_models.date_not_future],
        blank=True,
        null=True,
    )

    ltfu_date = models.DateField(
        verbose_name="Date lost to followup, if applicable",
        validators=[edc_models.date_not_future],
        blank=True,
        null=True,
    )

    transferred_consent = models.CharField(
        verbose_name=("If transferred, has the patient provided consent to be followed-up?"),
        choices=YES_NO_NA,
        max_length=15,
        default=NOT_APPLICABLE,
    )

    comment = models.TextField(verbose_name="Comments", null=True, blank=True)

    on_site = CurrentSiteManager()

    class Meta(OffScheduleModelMixin.Meta):
        verbose_name = "End of Study"
        verbose_name_plural = "End of Study"



# Generated by Django 3.0.9 on 2020-12-16 01:17

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("mocca_screening", "0007_auto_20201215_1552"),
    ]

    operations = [
        migrations.AlterField(
            model_name="carestatus",
            name="icc_not_in_reason",
            field=models.CharField(
                choices=[
                    (
                        "icc_not_available",
                        "ICC not available (or closed) in this facility",
                    ),
                    ("moved", "Moved out of area"),
                    (
                        "dont_want",
                        "Personally chose not to continue with integrated care",
                    ),
                    (
                        "advised_to_vertical",
                        "Healthcare staff asked patient to return to vertical care",
                    ),
                    ("referred_out", "Referred to another facility without an ICC"),
                    (
                        "receives_community_care",
                        "Patient is currently receiving their HIV care in the community",
                    ),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=25,
                verbose_name="If not integrated care, why not?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalcarestatus",
            name="icc_not_in_reason",
            field=models.CharField(
                choices=[
                    (
                        "icc_not_available",
                        "ICC not available (or closed) in this facility",
                    ),
                    ("moved", "Moved out of area"),
                    (
                        "dont_want",
                        "Personally chose not to continue with integrated care",
                    ),
                    (
                        "advised_to_vertical",
                        "Healthcare staff asked patient to return to vertical care",
                    ),
                    ("referred_out", "Referred to another facility without an ICC"),
                    (
                        "receives_community_care",
                        "Patient is currently receiving their HIV care in the community",
                    ),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=25,
                verbose_name="If not integrated care, why not?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectrefusal",
            name="reason",
            field=models.CharField(
                choices=[
                    ("unwilling_to_say", "Unwilling to give any reason"),
                    (
                        "worried_about_care",
                        "Worried that he/she will not get the care needed",
                    ),
                    (
                        "mixing_and_stigme",
                        "Does not want to mix with patients with other conditions because of concern around stigma",
                    ),
                    (
                        "mixing_and_infection",
                        "Does not want to mix with patients with other conditions because of concerns around contracting infections",
                    ),
                    (
                        "happy_vertical_care",
                        "Patient is happy with the care being given in vertical clinics",
                    ),
                    (
                        "no_participant",
                        "No longer wishes to be a study participant but is happy to continue receiving care in the integrated clinic",
                    ),
                    (
                        "receives_community_care",
                        "Patient is receiving HIV care in the community and does not wish to return to facility-based care",
                    ),
                    ("OTHER", "Other, please specify"),
                ],
                max_length=25,
                verbose_name="Reason for refusal to join",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectrefusalscreening",
            name="reason",
            field=models.CharField(
                choices=[
                    ("unwilling_to_say", "Unwilling to give any reason"),
                    (
                        "worried_about_care",
                        "Worried that he/she will not get the care needed",
                    ),
                    (
                        "mixing_and_stigme",
                        "Does not want to mix with patients with other conditions because of concern around stigma",
                    ),
                    (
                        "mixing_and_infection",
                        "Does not want to mix with patients with other conditions because of concerns around contracting infections",
                    ),
                    (
                        "happy_vertical_care",
                        "Patient is happy with the care being given in vertical clinics",
                    ),
                    (
                        "no_participant",
                        "No longer wishes to be a study participant but is happy to continue receiving care in the integrated clinic",
                    ),
                    (
                        "receives_community_care",
                        "Patient is receiving HIV care in the community and does not wish to return to facility-based care",
                    ),
                    ("OTHER", "Other, please specify"),
                ],
                max_length=25,
                verbose_name="Reason for refusal to screen",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="icc_not_in_reason",
            field=models.CharField(
                choices=[
                    (
                        "icc_not_available",
                        "ICC not available (or closed) in this facility",
                    ),
                    ("moved", "Moved out of area"),
                    (
                        "dont_want",
                        "Personally chose not to continue with integrated care",
                    ),
                    (
                        "advised_to_vertical",
                        "Healthcare staff asked patient to return to vertical care",
                    ),
                    ("referred_out", "Referred to another facility without an ICC"),
                    (
                        "receives_community_care",
                        "Patient is currently receiving their HIV care in the community",
                    ),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=25,
                verbose_name="If not integrated care, why not?",
            ),
        ),
        migrations.AlterField(
            model_name="subjectrefusal",
            name="reason",
            field=models.CharField(
                choices=[
                    ("unwilling_to_say", "Unwilling to give any reason"),
                    (
                        "worried_about_care",
                        "Worried that he/she will not get the care needed",
                    ),
                    (
                        "mixing_and_stigme",
                        "Does not want to mix with patients with other conditions because of concern around stigma",
                    ),
                    (
                        "mixing_and_infection",
                        "Does not want to mix with patients with other conditions because of concerns around contracting infections",
                    ),
                    (
                        "happy_vertical_care",
                        "Patient is happy with the care being given in vertical clinics",
                    ),
                    (
                        "no_participant",
                        "No longer wishes to be a study participant but is happy to continue receiving care in the integrated clinic",
                    ),
                    (
                        "receives_community_care",
                        "Patient is receiving HIV care in the community and does not wish to return to facility-based care",
                    ),
                    ("OTHER", "Other, please specify"),
                ],
                max_length=25,
                verbose_name="Reason for refusal to join",
            ),
        ),
        migrations.AlterField(
            model_name="subjectrefusalscreening",
            name="reason",
            field=models.CharField(
                choices=[
                    ("unwilling_to_say", "Unwilling to give any reason"),
                    (
                        "worried_about_care",
                        "Worried that he/she will not get the care needed",
                    ),
                    (
                        "mixing_and_stigme",
                        "Does not want to mix with patients with other conditions because of concern around stigma",
                    ),
                    (
                        "mixing_and_infection",
                        "Does not want to mix with patients with other conditions because of concerns around contracting infections",
                    ),
                    (
                        "happy_vertical_care",
                        "Patient is happy with the care being given in vertical clinics",
                    ),
                    (
                        "no_participant",
                        "No longer wishes to be a study participant but is happy to continue receiving care in the integrated clinic",
                    ),
                    (
                        "receives_community_care",
                        "Patient is receiving HIV care in the community and does not wish to return to facility-based care",
                    ),
                    ("OTHER", "Other, please specify"),
                ],
                max_length=25,
                verbose_name="Reason for refusal to screen",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="icc_not_in_reason",
            field=models.CharField(
                choices=[
                    (
                        "icc_not_available",
                        "ICC not available (or closed) in this facility",
                    ),
                    ("moved", "Moved out of area"),
                    (
                        "dont_want",
                        "Personally chose not to continue with integrated care",
                    ),
                    (
                        "advised_to_vertical",
                        "Healthcare staff asked patient to return to vertical care",
                    ),
                    ("referred_out", "Referred to another facility without an ICC"),
                    (
                        "receives_community_care",
                        "Patient is currently receiving their HIV care in the community",
                    ),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=25,
                verbose_name="If not integrated care, why not?",
            ),
        ),
    ]



from bs4 import BeautifulSoup
from django import template
from django.conf import settings
from edc_constants.constants import NO, TBD, YES
from edc_dashboard.url_names import url_names
from edc_screening import eligibility_display_label

register = template.Library()


@register.inclusion_tag(
    f"mocca_dashboard/bootstrap{settings.EDC_BOOTSTRAP}/" f"buttons/screening_button.html",
    takes_context=True,
)
def screening_button(context, model_wrapper):
    title = "Edit subject's screening form"
    return dict(
        perms=context["perms"],
        screening_identifier=model_wrapper.object.screening_identifier,
        href=model_wrapper.href,
        title=title,
        YES=YES,
        NO=NO,
        TBD=TBD,
    )


@register.inclusion_tag(
    f"mocca_dashboard/bootstrap{settings.EDC_BOOTSTRAP}/" f"buttons/eligibility_button.html"
)
def eligibility_button(subject_screening_model_wrapper):
    comment = []
    obj = subject_screening_model_wrapper.object
    tooltip = None
    if not obj.eligible and obj.reasons_ineligible:
        comment = obj.reasons_ineligible.split("|")
        comment = list(set(comment))
        comment.sort()
    soup = BeautifulSoup(eligibility_display_label(obj.eligible), features="html.parser")
    return dict(
        eligible=obj.eligible,
        display_label=soup.get_text(),
        comment=comment,
        tooltip=tooltip,
        TBD=TBD,
    )


@register.inclusion_tag(
    f"mocca_dashboard/bootstrap{settings.EDC_BOOTSTRAP}/buttons/add_consent_button.html",
    takes_context=True,
)
def add_consent_button(context, model_wrapper):
    title = ["Consent subject to participate."]
    consent_version = model_wrapper.consent.version
    return dict(
        perms=context["perms"],
        screening_identifier=model_wrapper.object.screening_identifier,
        href=model_wrapper.consent.href,
        consent_version=consent_version,
        title=" ".join(title),
    )


@register.inclusion_tag(
    f"mocca_dashboard/bootstrap{settings.EDC_BOOTSTRAP}/buttons/refusal_button.html",
    takes_context=True,
)
def refusal_button(context, model_wrapper):
    title = ["Capture subject's primary reason for not joining."]
    return dict(
        perms=context["perms"],
        screening_identifier=model_wrapper.object.screening_identifier,
        href=model_wrapper.refusal.href,
        title=" ".join(title),
    )


@register.inclusion_tag(
    f"mocca_dashboard/bootstrap{settings.EDC_BOOTSTRAP}/" f"buttons/dashboard_button.html"
)
def dashboard_button(model_wrapper):
    subject_dashboard_url = url_names.get("subject_dashboard_url")
    return dict(
        subject_dashboard_url=subject_dashboard_url,
        subject_identifier=model_wrapper.subject_identifier,
    )



from django import forms
from edc_constants.constants import OTHER, YES
from edc_dx_review.utils import medications_exists_or_raise
from edc_visit_schedule.utils import is_baseline


class DrugRefillFormValidatorMixin:
    def drug_refill_clean(self) -> None:
        medications_exists_or_raise(self.cleaned_data.get("subject_visit"))
        if (
            self.cleaned_data.get("subject_visit")
            and is_baseline(self.cleaned_data.get("subject_visit"))
            and self.cleaned_data.get("rx_modified") == YES
        ):
            raise forms.ValidationError({"rx_modified": "Expected `No` at baseline."})

        self.m2m_other_specify(
            OTHER, m2m_field="modifications", field_other="modifications_other"
        )
        self.m2m_other_specify(
            OTHER,
            m2m_field="modification_reasons",
            field_other="modification_reasons_other",
        )



from django.contrib.admin import AdminSite as DjangoAdminSite
from django.contrib.sites.shortcuts import get_current_site
from edc_sites.models import SiteProfile


class AdminSite(DjangoAdminSite):
    site_title = "MOCCA Screening CRFs"
    site_header = "MOCCA Screening CRFs"
    index_title = "MOCCA Screening CRFs"
    site_url = "/administration/"
    enable_nav_sidebar = False  # DJ 3.1

    def each_context(self, request):
        context = super().each_context(request)
        title = SiteProfile.objects.get(site=get_current_site(request)).title
        context.update(global_site=get_current_site(request))
        label = f"MOCCA: {title.title()} - Screening CRFs"
        context.update(site_title=label, site_header=label, index_title=label)
        return context


mocca_screening_admin = AdminSite(name="mocca_screening_admin")
mocca_screening_admin.disable_action("delete_selected")



from django import forms
from edc_crf.forms import CrfFormValidatorMixin
from edc_crf.modelform_mixins import CrfModelFormMixin
from edc_dx_review.utils import (
    raise_if_both_ago_and_actual_date,
    raise_if_clinical_review_does_not_exist,
)
from edc_form_validators.form_validator import FormValidator
from edc_model.utils import estimated_date_from_ago

from ..constants import DRUGS
from ..models import CholInitialReview


class CholInitialReviewFormValidator(
    CrfFormValidatorMixin,
    FormValidator,
):
    def clean(self):
        raise_if_clinical_review_does_not_exist(self.cleaned_data.get("subject_visit"))
        raise_if_both_ago_and_actual_date(cleaned_data=self.cleaned_data)

        # TODO: How is CHOL managed? Like DM? Drugs and lifestyle?
        self.required_if(
            DRUGS,
            field="managed_by",
            field_required="med_start_ago",
        )

        if self.cleaned_data.get("dx_ago") and self.cleaned_data.get("med_start_ago"):
            if (
                estimated_date_from_ago(data=self.cleaned_data, ago_field="dx_ago")
                - estimated_date_from_ago(data=self.cleaned_data, ago_field="med_start_ago")
            ).days > 1:
                raise forms.ValidationError(
                    {"med_start_ago": "Invalid. Cannot be before diagnosis."}
                )


class CholInitialReviewForm(CrfModelFormMixin, forms.ModelForm):
    form_validator_cls = CholInitialReviewFormValidator

    class Meta:
        model = CholInitialReview
        fields = "__all__"



from edc_reportable import GRADE3, GRADE4, site_reportables
from edc_reportable.grading_data.daids_july_2017 import chemistries, dummies, hematology
from edc_reportable.normal_data.africa import normal_data

grading_data = {}
grading_data.update(**dummies)
grading_data.update(**chemistries)
grading_data.update(**hematology)

site_reportables.register(
    name="mocca",
    normal_data=normal_data,
    grading_data=grading_data,
    reportable_grades=[GRADE3, GRADE4],
    reportable_grades_exceptions=None,
)



from django.test import TestCase
from edc_lab.models import Panel
from edc_lab.site_labs import site_labs

from mocca_labs.lab_profiles import subject_lab_profile


class TestLabs(TestCase):
    def setUp(self):
        site_labs._registry = {}
        site_labs.loaded = False
        site_labs.register(lab_profile=subject_lab_profile)

    def test_(self):
        obj = site_labs.get(lab_profile_name="subject_lab_profile")
        self.assertEqual(obj, subject_lab_profile)

    def test_lab_profile_model(self):
        obj = site_labs.get(lab_profile_name="subject_lab_profile")
        self.assertEqual("mocca_subject.subjectrequisition", obj.requisition_model)

    def test_panel_model(self):
        for panel in site_labs.get(lab_profile_name="subject_lab_profile").panels.values():
            self.assertEqual(panel.requisition_model, "mocca_subject.subjectrequisition")

    def test_panels_exist(self):
        self.assertGreater(Panel.objects.all().count(), 0)



# Generated by Django 3.0.9 on 2020-12-15 03:47

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("mocca_screening", "0004_auto_20201215_0602"),
    ]

    operations = [
        migrations.AlterField(
            model_name="carestatus",
            name="icc",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                help_text="Either at this facility or elsewhere",
                max_length=25,
                verbose_name="Does the patient currently receive integrated care",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="icc",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                help_text="Either at this facility or elsewhere",
                max_length=25,
                verbose_name="Does the patient currently receive integrated care",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="icc",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                help_text="Either at this facility or elsewhere",
                max_length=25,
                verbose_name="Does the patient currently receive integrated care",
            ),
        ),
    ]



from .crf_model_mixin import CrfModelMixin
from .diagnosis_location_model_mixin import DiagnosisLocationModelMixin
from .medication_adherence_model_mixin import MedicationAdherenceModelMixin
from .search_slug_model_mixin import SearchSlugModelMixin
